@@ -1,8 +1,8 @@
 /* Copyright 2003-2004 Roger Dingledine
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson */
 /* See LICENSE for licensing information */
-/* $Id: compat.c,v 1.73 2005/10/07 19:33:47 nickm Exp $ */
-const char compat_c_id[] = "$Id: compat.c,v 1.73 2005/10/07 19:33:47 nickm Exp $";
+/* $Id: compat.c,v 1.77 2005/11/15 03:05:14 nickm Exp $ */
+const char compat_c_id[] = "$Id: compat.c,v 1.77 2005/11/15 03:05:14 nickm Exp $";
 
 /**
  * \file compat.c
@@ -177,6 +177,7 @@ tor_memmem(const void *_haystack, size_t hlen, const void *_needle, size_t nlen)
 #endif
 }
 
+#ifdef MS_WINDOWS
 /** Take a filename and return a pointer to its final element.  This
  * function is called on __FILE__ to fix a MSVC nit where __FILE__
  * contains the full path to the file.  This is bad, because it
@@ -184,7 +185,7 @@ tor_memmem(const void *_haystack, size_t hlen, const void *_needle, size_t nlen)
  * compiled the binary in their warrning messages.
  */
 const char *
-_tor_fix_source_file(const char *fname)
+tor_fix_source_file(const char *fname)
 {
   const char *cp1, *cp2, *r;
   cp1 = strrchr(fname, '/');
@@ -200,6 +201,7 @@ _tor_fix_source_file(const char *fname)
   }
   return r;
 }
+#endif
 
 #ifndef UNALIGNED_INT_ACCESS_OK
 /**
@@ -354,7 +356,7 @@ tor_socketpair(int family, int type, int protocol, int fd[2])
     if (listener == -1)
       return -tor_socket_errno(-1);
     if (!SOCKET_IS_POLLABLE(listener)) {
-      log_fn(LOG_WARN, "Too many connections; can't open socketpair");
+      warn(LD_NET, "Too many connections; can't open socketpair");
       tor_close_socket(listener);
 #ifdef MS_WINDOWS
       return -ENFILE;
@@ -376,7 +378,7 @@ tor_socketpair(int family, int type, int protocol, int fd[2])
     if (connector == -1)
       goto tidy_up_and_fail;
     if (!SOCKET_IS_POLLABLE(connector)) {
-      log_fn(LOG_WARN, "Too many connections; can't open socketpair");
+      warn(LD_NET, "Too many connections; can't open socketpair");
       goto tidy_up_and_fail;
     }
     /* We want to find out the port number to connect to.  */
@@ -394,7 +396,7 @@ tor_socketpair(int family, int type, int protocol, int fd[2])
     if (acceptor == -1)
       goto tidy_up_and_fail;
     if (!SOCKET_IS_POLLABLE(acceptor)) {
-      log_fn(LOG_WARN, "Too many connections; can't open socketpair");
+      warn(LD_NET, "Too many connections; can't open socketpair");
       goto tidy_up_and_fail;
     }
     if (size != sizeof(listen_addr))
@@ -443,9 +445,10 @@ int
 set_max_file_descriptors(unsigned long limit, unsigned long cap)
 {
 #ifndef HAVE_GETRLIMIT
-  log_fn(LOG_INFO,"This platform is missing getrlimit(). Proceeding.");
+  log_fn(LOG_INFO, LD_NET,
+         "This platform is missing getrlimit(). Proceeding.");
   if (limit > cap) {
-    log(LOG_INFO, "ConnLimit must be at most %d. Capping it.", cap);
+    log(LOG_INFO, LD_CONFIG, "ConnLimit must be at most %d. Capping it.", cap);
     limit = cap;
   }
 #else
@@ -455,22 +458,22 @@ set_max_file_descriptors(unsigned long limit, unsigned long cap)
   tor_assert(cap > 0);
 
   if (getrlimit(RLIMIT_NOFILE, &rlim) != 0) {
-    log_fn(LOG_WARN, "Could not get maximum number of file descriptors: %s",
+    warn(LD_NET, "Could not get maximum number of file descriptors: %s",
            strerror(errno));
     return -1;
   }
   if (rlim.rlim_max < limit) {
-    log_fn(LOG_WARN,"We need %lu file descriptors available, and we're limited to %lu. Please change your ulimit -n.", limit, (unsigned long)rlim.rlim_max);
+    warn(LD_CONFIG,"We need %lu file descriptors available, and we're limited to %lu. Please change your ulimit -n.", limit, (unsigned long)rlim.rlim_max);
     return -1;
   }
   most = (rlim.rlim_max > cap) ? cap : (unsigned) rlim.rlim_max;
   if (most > rlim.rlim_cur) {
-    log_fn(LOG_INFO,"Raising max file descriptors from %lu to %lu.",
+    info(LD_NET,"Raising max file descriptors from %lu to %lu.",
            (unsigned long)rlim.rlim_cur, most);
   }
   rlim.rlim_cur = most;
   if (setrlimit(RLIMIT_NOFILE, &rlim) != 0) {
-    log_fn(LOG_WARN, "Could not set maximum number of file descriptors: %s",
+    warn(LD_CONFIG, "Could not set maximum number of file descriptors: %s",
            strerror(errno));
     return -1;
   }
@@ -479,7 +482,7 @@ set_max_file_descriptors(unsigned long limit, unsigned long cap)
 #endif
 
   if (limit < ULIMIT_BUFFER) {
-    log_fn(LOG_WARN,"ConnLimit must be at least %d. Failing.", ULIMIT_BUFFER);
+    warn(LD_CONFIG,"ConnLimit must be at least %d. Failing.", ULIMIT_BUFFER);
     return -1;
   }
   return limit - ULIMIT_BUFFER;
@@ -498,7 +501,7 @@ switch_id(char *user, char *group)
   if (user) {
     pw = getpwnam(user);
     if (pw == NULL) {
-      log_fn(LOG_ERR,"User '%s' not found.", user);
+      err(LD_CONFIG,"User '%s' not found.", user);
       return -1;
     }
   }
@@ -507,17 +510,17 @@ switch_id(char *user, char *group)
   if (group) {
     gr = getgrnam(group);
     if (gr == NULL) {
-      log_fn(LOG_ERR,"Group '%s' not found.", group);
+      err(LD_CONFIG,"Group '%s' not found.", group);
       return -1;
     }
 
     if (setgid(gr->gr_gid) != 0) {
-      log_fn(LOG_ERR,"Error setting GID: %s", strerror(errno));
+      err(LD_GENERAL,"Error setting GID: %s", strerror(errno));
       return -1;
     }
   } else if (user) {
     if (setgid(pw->pw_gid) != 0) {
-      log_fn(LOG_ERR,"Error setting GID: %s", strerror(errno));
+      err(LD_GENERAL,"Error setting GID: %s", strerror(errno));
       return -1;
     }
   }
@@ -526,7 +529,7 @@ switch_id(char *user, char *group)
      privileges */
   if (user) {
     if (setuid(pw->pw_uid) != 0) {
-      log_fn(LOG_ERR,"Error setting UID: %s", strerror(errno));
+      err(LD_GENERAL,"Error setting UID: %s", strerror(errno));
       return -1;
     }
   }
@@ -534,9 +537,7 @@ switch_id(char *user, char *group)
   return 0;
 #endif
 
-  log_fn(LOG_ERR,
-         "User or group specified, but switching users is not supported.");
-
+  err(LD_CONFIG,"User or group specified, but switching users is not supported.");
   return -1;
 }
 
@@ -550,7 +551,7 @@ get_user_homedir(const char *username)
   tor_assert(username);
 
   if (!(pw = getpwnam(username))) {
-    log_fn(LOG_ERR,"User \"%s\" not found.", username);
+    err(LD_CONFIG,"User \"%s\" not found.", username);
     return NULL;
   }
   return tor_strdup(pw->pw_dir);
@@ -695,6 +696,7 @@ get_uname(void)
 #ifdef MS_WINDOWS
         OSVERSIONINFOEX info;
         int i;
+        unsigned int leftover_mask;
         const char *plat = NULL;
         static struct {
           int major; int minor; const char *version;
@@ -710,6 +712,21 @@ get_uname(void)
           { 3, 51, "Windows NT 3.51" },
           { -1, -1, NULL }
         };
+        static struct {
+          unsigned int mask; const char *str;
+        } win_mask_table[] = {
+          { VER_SUITE_BACKOFFICE,         " {backoffice}" },
+          { VER_SUITE_BLADE,              " {\"blade\" (2003, web edition)}" },
+          { VER_SUITE_DATACENTER,         " {datacenter}" },
+          { VER_SUITE_ENTERPRISE,         " {enterprise}" },
+          { VER_SUITE_EMBEDDEDNT,         " {embedded}" },
+          { VER_SUITE_PERSONAL,           " {personal}" },
+          { VER_SUITE_SINGLEUSERTS,       " {terminal services, single user}" },
+          { VER_SUITE_SMALLBUSINESS,      " {small business}" },
+          { VER_SUITE_SMALLBUSINESS_RESTRICTED, " {small business, restricted}" },
+          { VER_SUITE_TERMINAL,           " {terminal services}" },
+          { 0, NULL },
+        };
         info.dwOSVersionInfoSize = sizeof(info);
         GetVersionEx((LPOSVERSIONINFO)&info);
         if (info.dwMajorVersion == 4 && info.dwMinorVersion == 0) {
@@ -733,12 +750,14 @@ get_uname(void)
           if (info.dwMajorVersion > 6 ||
               (info.dwMajorVersion==6 && info.dwMinorVersion>0))
             tor_snprintf(uname_result, sizeof(uname_result),
-                         "Very recent version of Windows [major=%d,minor=%d]",
-                         (int)info.dwMajorVersion,(int)info.dwMinorVersion);
+                         "Very recent version of Windows [major=%d,minor=%d] %s",
+                         (int)info.dwMajorVersion,(int)info.dwMinorVersion,
+                         info.szCSDVersion);
           else
             tor_snprintf(uname_result, sizeof(uname_result),
-                         "Unrecognized version of Windows [major=%d,minor=%d]",
-                         (int)info.dwMajorVersion,(int)info.dwMinorVersion);
+                         "Unrecognized version of Windows [major=%d,minor=%d] %s",
+                         (int)info.dwMajorVersion,(int)info.dwMinorVersion,
+                         info.szCSDVersion);
         }
         if (info.wProductType == VER_NT_DOMAIN_CONTROLLER) {
           strlcat(uname_result, " [domain controller]", sizeof(uname_result));
@@ -747,20 +766,17 @@ get_uname(void)
         } else if (info.wProductType == VER_NT_WORKSTATION) {
           strlcat(uname_result, " [workstation]", sizeof(uname_result));
         }
-        if (info.wSuiteMask & VER_SUITE_DATACENTER) {
-          strlcat(uname_result, " {datacenter}", sizeof(uname_result));
+        leftover_mask = info.wSuiteMask;
+        for (i = 0; win_mask_table[i].mask; ++i) {
+          if (info.wSuiteMask & win_mask_table[i].mask) {
+            strlcat(uname_result, win_mask_table[i].str, sizeof(uname_result));
+            leftover_mask &= ~win_mask_table[i].mask;
+          }
         }
-        if (info.wSuiteMask & VER_SUITE_ENTERPRISE) {
-          strlcat(uname_result, " {enterprise}", sizeof(uname_result));
-        }
-        if (info.wSuiteMask & VER_SUITE_PERSONAL) {
-          strlcat(uname_result, " {personal}", sizeof(uname_result));
-        }
-        if (info.wSuiteMask & VER_SUITE_EMBEDDEDNT) {
-          strlcat(uname_result, " {personal}", sizeof(uname_result));
-        }
-        if (info.wSuiteMask & VER_SUITE_PERSONAL) {
-          strlcat(uname_result, " {personal}", sizeof(uname_result));
+        if (leftover_mask) {
+          size_t len = strlen(uname_result);
+          tor_snprintf(uname_result+len, sizeof(uname_result)-len,
+                       " {0x%x}", info.wSuiteMask);
         }
 #else
         strlcpy(uname_result, "Unknown platform", sizeof(uname_result));
@@ -879,7 +895,7 @@ tor_gettimeofday(struct timeval *timeval)
   /* number of 100-nsec units since Jan 1, 1601 */
   GetSystemTimeAsFileTime(&ft.ft_ft);
   if (ft.ft_64 < EPOCH_BIAS) {
-    log_fn(LOG_ERR, "System time is before 1970; failing.");
+    err(LD_GENERAL,"System time is before 1970; failing.");
     exit(1);
   }
   ft.ft_64 -= EPOCH_BIAS;
@@ -887,7 +903,7 @@ tor_gettimeofday(struct timeval *timeval)
   timeval->tv_usec = (unsigned) ((ft.ft_64 / UNITS_PER_USEC) % USEC_PER_SEC);
 #elif defined(HAVE_GETTIMEOFDAY)
   if (gettimeofday(timeval, NULL)) {
-    log_fn(LOG_ERR, "gettimeofday failed.");
+    err(LD_GENERAL,"gettimeofday failed.");
     /* If gettimeofday dies, we have either given a bad timezone (we didn't),
        or segfaulted.*/
     exit(1);
@@ -996,7 +1012,7 @@ tor_mutex_acquire(tor_mutex_t *m)
       tor_assert(0);
       break;
     case WAIT_FAILED:
-      log_fn(LOG_WARN, "Failed to acquire mutex: %d", GetLastError());
+      warn(LD_GENERAL, "Failed to acquire mutex: %d", GetLastError());
   }
 }
 void
@@ -1005,7 +1021,7 @@ tor_mutex_release(tor_mutex_t *m)
   BOOL r;
   r = ReleaseMutex(m->handle);
   if (!r) {
-    log_fn(LOG_WARN, "Failed to release mutex: %d", GetLastError());
+    warn(LD_GENERAL, "Failed to release mutex: %d", GetLastError());
   }
 }
 unsigned long
@@ -1177,7 +1193,7 @@ network_init(void)
   int r;
   r = WSAStartup(0x101,&WSAData);
   if (r) {
-    log_fn(LOG_WARN,"Error initializing windows network layer: code was %d",r);
+    warn(LD_NET,"Error initializing windows network layer: code was %d",r);
     return -1;
   }
   /* WSAData.iMaxSockets might show the max sockets we're allowed to use.
