@@ -1,8 +1,8 @@
 /* Copyright 2004 Roger Dingledine */
 /* Copyright 2004-2005 Roger Dingledine, Nick Mathewson */
 /* See LICENSE for licensing information */
-/* $Id: torgzip.c,v 1.22 2005/09/30 01:09:52 nickm Exp $ */
-const char torgzip_c_id[] = "$Id: torgzip.c,v 1.22 2005/09/30 01:09:52 nickm Exp $";
+/* $Id: torgzip.c,v 1.24 2005/10/18 21:58:19 nickm Exp $ */
+const char torgzip_c_id[] = "$Id: torgzip.c,v 1.24 2005/10/18 21:58:19 nickm Exp $";
 
 /**
  * \file torgzip.c
@@ -75,7 +75,7 @@ tor_gzip_compress(char **out, size_t *out_len,
 
   if (method == GZIP_METHOD && !is_gzip_supported()) {
     /* Old zlib version don't support gzip in deflateInit2 */
-    log_fn(LOG_WARN, "Gzip not supported with zlib %s", ZLIB_VERSION);
+    warn(LD_GENERAL, "Gzip not supported with zlib %s", ZLIB_VERSION);
     return -1;
   }
 
@@ -91,7 +91,7 @@ tor_gzip_compress(char **out, size_t *out_len,
   if (deflateInit2(stream, Z_BEST_COMPRESSION, Z_DEFLATED,
                    method_bits(method),
                    8, Z_DEFAULT_STRATEGY) != Z_OK) {
-    log_fn(LOG_WARN, "Error from deflateInit2: %s",
+    warn(LD_GENERAL, "Error from deflateInit2: %s",
            stream->msg?stream->msg:"<no message>");
     goto err;
   }
@@ -120,7 +120,7 @@ tor_gzip_compress(char **out, size_t *out_len,
         stream->avail_out = out_size - offset;
         break;
       default:
-        log_fn(LOG_WARN, "Gzip compression didn't finish: %s",
+        warn(LD_GENERAL, "Gzip compression didn't finish: %s",
                stream->msg ? stream->msg : "<no message>");
         goto err;
       }
@@ -128,7 +128,7 @@ tor_gzip_compress(char **out, size_t *out_len,
  done:
   *out_len = stream->total_out;
   if (deflateEnd(stream)!=Z_OK) {
-    log_fn(LOG_WARN, "Error freeing gzip structures");
+    warn(LD_GENERAL, "Error freeing gzip structures");
     goto err;
   }
   tor_free(stream);
@@ -154,7 +154,8 @@ tor_gzip_compress(char **out, size_t *out_len,
 int
 tor_gzip_uncompress(char **out, size_t *out_len,
                     const char *in, size_t in_len,
-                    compress_method_t method)
+                    compress_method_t method,
+                    int complete_only)
 {
   struct z_stream_s *stream = NULL;
   size_t out_size;
@@ -167,7 +168,7 @@ tor_gzip_uncompress(char **out, size_t *out_len,
 
   if (method == GZIP_METHOD && !is_gzip_supported()) {
     /* Old zlib version don't support gzip in inflateInit2 */
-    log_fn(LOG_WARN, "Gzip not supported with zlib %s", ZLIB_VERSION);
+    warn(LD_GENERAL, "Gzip not supported with zlib %s", ZLIB_VERSION);
     return -1;
   }
 
@@ -182,7 +183,7 @@ tor_gzip_uncompress(char **out, size_t *out_len,
 
   if (inflateInit2(stream,
                    method_bits(method)) != Z_OK) {
-    log_fn(LOG_WARN, "Error from inflateInit2: %s",
+    warn(LD_GENERAL, "Error from inflateInit2: %s",
            stream->msg?stream->msg:"<no message>");
     goto err;
   }
@@ -195,22 +196,29 @@ tor_gzip_uncompress(char **out, size_t *out_len,
   stream->avail_out = out_size;
 
   while (1) {
-    switch (inflate(stream, Z_FINISH))
+    switch (inflate(stream, complete_only ? Z_FINISH : Z_SYNC_FLUSH))
       {
       case Z_STREAM_END:
         if (stream->avail_in == 0)
           goto done;
+        /* There may be more compressed data here. */
         if (inflateInit2(stream, method_bits(method)) != Z_OK) {
-          log_fn(LOG_WARN, "Error from inflateInit2: %s",
+          warn(LD_GENERAL, "Error from inflateInit2: %s",
                  stream->msg?stream->msg:"<no message>");
           goto err;
         }
         break;
       case Z_OK:
+        if (!complete_only && stream->avail_in == 0)
+          goto done;
         /* In case zlib doesn't work as I think.... */
         if (stream->avail_out >= stream->avail_in+16)
           break;
       case Z_BUF_ERROR:
+        if (stream->avail_out > 0) {
+          warn(LD_PROTOCOL, "possible truncated or corrupt zlib data");
+          goto err;
+        }
         offset = stream->next_out - (unsigned char*)*out;
         out_size *= 2;
         *out = tor_realloc(*out, out_size);
@@ -218,7 +226,7 @@ tor_gzip_uncompress(char **out, size_t *out_len,
         stream->avail_out = out_size - offset;
         break;
       default:
-        log_fn(LOG_WARN, "Gzip decompression returned an error: %s",
+        warn(LD_GENERAL, "Gzip decompression returned an error: %s",
                stream->msg ? stream->msg : "<no message>");
         goto err;
       }
@@ -228,7 +236,7 @@ tor_gzip_uncompress(char **out, size_t *out_len,
   r = inflateEnd(stream);
   tor_free(stream);
   if (r != Z_OK) {
-    log_fn(LOG_WARN, "Error freeing gzip structures");
+    warn(LD_GENERAL, "Error freeing gzip structures");
     goto err;
   }
 
