@@ -1,7 +1,7 @@
 /* Copyright 2003 Roger Dingledine
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson */
 /* See LICENSE for licensing information */
-/* $Id: torgzip.h,v 1.7 2005/06/09 19:03:31 nickm Exp $ */
+/* $Id: torgzip.h,v 1.8 2005/10/13 22:48:09 nickm Exp $ */
 
 /**
  * \file torgzip.h
@@ -10,7 +10,7 @@
 
 #ifndef __TORGZIP_H
 #define __TORGZIP_H
-#define TORGZIP_H_ID "$Id: torgzip.h,v 1.7 2005/06/09 19:03:31 nickm Exp $"
+#define TORGZIP_H_ID "$Id: torgzip.h,v 1.8 2005/10/13 22:48:09 nickm Exp $"
 
 typedef enum {
   GZIP_METHOD=1, ZLIB_METHOD=2, UNKNOWN_METHOD=3
@@ -23,7 +23,8 @@ tor_gzip_compress(char **out, size_t *out_len,
 int
 tor_gzip_uncompress(char **out, size_t *out_len,
                     const char *in, size_t in_len,
-                    compress_method_t method);
+                    compress_method_t method,
+                    int complete_only);
 
 int is_gzip_supported(void);
 
