@@ -1,11 +1,11 @@
 /* Copyright 2003 Roger Dingledine
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson */
 /* See LICENSE for licensing information */
-/* $Id: tortls.h,v 1.30 2005/10/06 04:33:40 nickm Exp $ */
+/* $Id: tortls.h,v 1.32 2005/10/25 07:05:03 nickm Exp $ */
 
 #ifndef _TORTLS_H
 #define _TORTLS_H
-#define TORTLS_H_ID "$Id: tortls.h,v 1.30 2005/10/06 04:33:40 nickm Exp $"
+#define TORTLS_H_ID "$Id: tortls.h,v 1.32 2005/10/25 07:05:03 nickm Exp $"
 
 /**
  * \file tortls.h
@@ -33,7 +33,7 @@ int tor_tls_is_server(tor_tls_t *tls);
 void tor_tls_free(tor_tls_t *tls);
 int tor_tls_peer_has_cert(tor_tls_t *tls);
 int tor_tls_get_peer_cert_nickname(tor_tls_t *tls, char *buf, size_t buflen);
-int tor_tls_verify(tor_tls_t *tls, crypto_pk_env_t **identity);
+int tor_tls_verify(int severity, tor_tls_t *tls, crypto_pk_env_t **identity);
 int tor_tls_check_lifetime(tor_tls_t *tls, int tolerance);
 int tor_tls_read(tor_tls_t *tls, char *cp, size_t len);
 int tor_tls_write(tor_tls_t *tls, char *cp, size_t n);
@@ -46,7 +46,7 @@ unsigned long tor_tls_get_n_bytes_written(tor_tls_t *tls);
 
 /* Log and abort if there are unhandled TLS errors in OpenSSL's error stack.
  */
-#define check_no_tls_errors() _check_no_tls_errors(_SHORT_FILE_,__LINE__)
+#define check_no_tls_errors() _check_no_tls_errors(__FILE__,__LINE__)
 
 void _check_no_tls_errors(const char *fname, int line);
 
