@@ -1,7 +1,7 @@
 /* Copyright 2003-2004 Roger Dingledine
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson */
 /* See LICENSE for licensing information */
-/* $Id: util.h,v 1.143 2005/10/06 04:33:40 nickm Exp $ */
+/* $Id: util.h,v 1.148 2005/11/01 06:12:51 arma Exp $ */
 
 /**
  * \file util.h
@@ -10,12 +10,13 @@
 
 #ifndef __UTIL_H
 #define __UTIL_H
-#define UTIL_H_ID "$Id: util.h,v 1.143 2005/10/06 04:33:40 nickm Exp $"
+#define UTIL_H_ID "$Id: util.h,v 1.148 2005/11/01 06:12:51 arma Exp $"
 
 #include "orconfig.h"
 #include "torint.h"
 #include "compat.h"
 #include <stdio.h>
+#include <stdlib.h>
 #ifdef HAVE_SYS_TIME_H
 #include <sys/time.h>
 #endif
@@ -37,14 +38,14 @@
  */
 #error "Sorry; we don't support building with NDEBUG."
 #else
-#define tor_assert(expr) do {                                 \
- if (!(expr)) {                                               \
-   log(LOG_ERR, "%s:%d: %s: Assertion %s failed; aborting.",  \
-       _SHORT_FILE_, __LINE__, __FUNCTION__, #expr);          \
-   fprintf(stderr,"%s:%d %s: Assertion %s failed; aborting.\n", \
-       _SHORT_FILE_, __LINE__, __FUNCTION__, #expr);          \
-   abort();  /* unreached */                                  \
- } } while (0)
+#define tor_assert(expr) do {                                           \
+    if (!(expr)) {                                                      \
+      log(LOG_ERR, LD_BUG, "%s:%d: %s: Assertion %s failed; aborting.", \
+          _SHORT_FILE_, __LINE__, __FUNCTION__, #expr);                 \
+      fprintf(stderr,"%s:%d %s: Assertion %s failed; aborting.\n",      \
+              _SHORT_FILE_, __LINE__, __FUNCTION__, #expr);             \
+      abort();                                                          \
+    } } while (0)
 #endif
 
 #ifdef USE_DMALLOC
