@@ -2,8 +2,8 @@
  * Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: circuitlist.c,v 1.60 2005/10/06 22:18:01 nickm Exp $ */
-const char circuitlist_c_id[] = "$Id: circuitlist.c,v 1.60 2005/10/06 22:18:01 nickm Exp $";
+/* $Id: circuitlist.c,v 1.70 2005/11/11 19:28:32 arma Exp $ */
+const char circuitlist_c_id[] = "$Id: circuitlist.c,v 1.70 2005/11/11 19:28:32 arma Exp $";
 
 /**
  * \file circuitlist.c
@@ -99,7 +99,7 @@ circuit_set_circid_orconn(circuit_t *circ, uint16_t id,
     _last_circid_orconn_ent = NULL;
   }
 
-  if (old_conn) {
+  if (old_conn) { /* we may need to remove it from the conn-circid map */
     search.circ_id = old_id;
     search.or_conn = old_conn;
     found = RB_FIND(orconn_circid_tree, &orconn_circid_circuit_map, &search);
@@ -112,6 +112,7 @@ circuit_set_circid_orconn(circuit_t *circ, uint16_t id,
   if (conn == NULL)
     return;
 
+  /* now add the new one to the conn-circid map */
   search.circ_id = id;
   search.or_conn = conn;
   found = RB_FIND(orconn_circid_tree, &orconn_circid_circuit_map, &search);
@@ -187,7 +188,7 @@ circuit_state_to_string(int state)
     case CIRCUIT_STATE_OR_WAIT: return "connecting to firsthop";
     case CIRCUIT_STATE_OPEN: return "open";
     default:
-      log_fn(LOG_WARN, "Bug: unknown circuit state %d", state);
+      warn(LD_BUG, "Bug: unknown circuit state %d", state);
       tor_snprintf(buf, sizeof(buf), "unknown state [%d]", state);
       return buf;
   }
@@ -201,7 +202,8 @@ circuit_new(uint16_t p_circ_id, connection_t *p_conn)
 {
   circuit_t *circ;
   static uint32_t n_circuits_allocated = 1;
-    /* never zero, since a global ID of 0 is treated specially by the controller */
+  /* never zero, since a global ID of 0 is treated specially by the
+   * controller */
 
   circ = tor_malloc_zero(sizeof(circuit_t));
   circ->magic = CIRCUIT_MAGIC;
@@ -249,6 +251,7 @@ circuit_free(circuit_t *circ)
       circuit_free_cpath_node(circ->build_state->pending_final_cpath);
   }
   tor_free(circ->build_state);
+  tor_free(circ->onionskin);
   circuit_free_cpath(circ->cpath);
   if (circ->rend_splice) {
     circ->rend_splice->rend_splice = NULL;
@@ -339,12 +342,11 @@ circuit_get_by_global_id(uint32_t id)
 
 /** Return a circ such that:
  *  - circ-\>n_circ_id or circ-\>p_circ_id is equal to <b>circ_id</b>, and
- *  - circ is attached to <b>conn</b>, either as p_conn, n-conn, or
- *    in p_streams or n_streams.
+ *  - circ is attached to <b>conn</b>, either as p_conn or n_conn.
  * Return NULL if no such circuit exists.
  */
-circuit_t *
-circuit_get_by_circid_orconn(uint16_t circ_id, connection_t *conn)
+static INLINE circuit_t *
+circuit_get_by_circid_orconn_impl(uint16_t circ_id, connection_t *conn)
 {
   orconn_circid_circuit_map_t search;
   orconn_circid_circuit_map_t *found;
@@ -361,30 +363,58 @@ circuit_get_by_circid_orconn(uint16_t circ_id, connection_t *conn)
     found = RB_FIND(orconn_circid_tree, &orconn_circid_circuit_map, &search);
     _last_circid_orconn_ent = found;
   }
-  if (found && found->circuit && !found->circuit->marked_for_close)
+  if (found && found->circuit)
     return found->circuit;
 
-  /* The rest of this can be replaced with
-     "return NULL;" once we believe the code works. */
+  return NULL;
 
+  /* The rest of this checks for bugs. Disabled by default. */
   {
     circuit_t *circ;
     for (circ=global_circuitlist;circ;circ = circ->next) {
-      if (circ->marked_for_close)
-        continue;
-
       if (circ->p_conn == conn && circ->p_circ_id == circ_id) {
-        log_fn(LOG_WARN, "circuit matches p_conn, but not in tree (Bug!)");
+        warn(LD_BUG, "circuit matches p_conn, but not in tree (Bug!)");
         return circ;
       }
       if (circ->n_conn == conn && circ->n_circ_id == circ_id) {
-        log_fn(LOG_WARN, "circuit matches n_conn, but not in tree (Bug!)");
+        warn(LD_BUG, "circuit matches n_conn, but not in tree (Bug!)");
         return circ;
       }
     }
     return NULL;
   }
+}
 
+/** Return a circ such that:
+ *  - circ-\>n_circ_id or circ-\>p_circ_id is equal to <b>circ_id</b>, and
+ *  - circ is attached to <b>conn</b>, either as p_conn or n_conn.
+ *  - circ is not marked for close.
+ * Return NULL if no such circuit exists.
+ */
+circuit_t *
+circuit_get_by_circid_orconn(uint16_t circ_id, connection_t *conn)
+{
+  circuit_t *circ = circuit_get_by_circid_orconn_impl(circ_id, conn);
+  if (!circ || circ->marked_for_close)
+    return NULL;
+  else
+    return circ;
+}
+
+/** Return true iff there is a circ such that
+ *  - circ-\>n_circ_id or circ-\>p_circ_id is equal to <b>circ_id</b>, and
+ *  - circ is attached to <b>conn</b>, either as p_conn or n_conn.
+ * Return NULL if no such circuit exists.
+ */
+int
+circuit_id_used_on_conn(uint16_t circ_id, connection_t *conn)
+{
+  circuit_t *circ = circuit_get_by_circid_orconn_impl(circ_id, conn);
+  if (circ && circ->marked_for_close)
+    log_fn(LOG_NOTICE, LD_CIRC,
+           "I was about to re-use a circuit ID that had been marked."
+           " Good thing we fixed that bug!");
+  return circ != NULL;
 }
 
 /** Return the circuit that a given edge connection is using. */
@@ -401,7 +431,7 @@ circuit_get_by_edge_conn(connection_t *conn)
     /* return NULL; */
     circ = circuit_get_by_conn(conn);
     if (circ) {
-      log_fn(LOG_WARN, "BUG: conn->on_circuit==NULL, but there was in fact a circuit there.");
+      warn(LD_BUG, "BUG: conn->on_circuit==NULL, but there was in fact a circuit there.");
     }
     return circ;
   }
@@ -426,7 +456,7 @@ circuit_get_by_edge_conn(connection_t *conn)
 }
 
 /** Return a circ such that circ is attached to <b>conn</b>, either as
- * p_conn, n-conn, or in p_streams or n_streams or resolving_streams.
+ * p_conn, n_conn, or in p_streams or n_streams or resolving_streams.
  *
  * Return NULL if no such circuit exists.
  */
@@ -477,10 +507,10 @@ circuit_get_by_rend_query_and_purpose(const char *rend_query, uint8_t purpose)
   return NULL;
 }
 
-/** Return the first circuit in global_circuitlist after <b>start</b> whose
- * rend_pk_digest field is <b>digest</b> and whose purpose is <b>purpose</b>. Returns
- * NULL if no circuit is found.  If <b>start</b> is NULL, begin at the start of
- * the list.
+/** Return the first circuit in global_circuitlist after <b>start</b>
+ * whose rend_pk_digest field is <b>digest</b> and whose purpose is
+ * <b>purpose</b>. Returns NULL if no circuit is found.
+ * If <b>start</b> is NULL, begin at the start of the list.
  */
 circuit_t *
 circuit_get_next_by_pk_and_purpose(circuit_t *start,
@@ -520,21 +550,24 @@ circuit_get_rendezvous(const char *cookie)
 }
 
 /** Return a circuit that is open, has specified <b>purpose</b>,
- * has a timestamp_dirty value of 0, and is uptime/capacity/internal
- * if required; or NULL if no circuit fits this description.
+ * has a timestamp_dirty value of 0, is uptime/capacity/internal
+ * if required, and if info is defined, does not already use info
+ * as any of its hops; or NULL if no circuit fits this description.
  *
  * Avoid returning need_uptime circuits if not necessary.
+ *
  * FFFF As a more important goal, not yet implemented, avoid returning
  * internal circuits if not necessary.
  */
 circuit_t *
-circuit_get_clean_open(uint8_t purpose, int need_uptime,
-                       int need_capacity, int internal)
+circuit_find_to_cannibalize(uint8_t purpose, extend_info_t *info,
+                            int need_uptime,
+                            int need_capacity, int internal)
 {
   circuit_t *circ;
   circuit_t *best=NULL;
 
-  log_fn(LOG_DEBUG,"Hunting for a circ to cannibalize: purpose %d, uptime %d, capacity %d, internal %d", purpose, need_uptime, need_capacity, internal);
+  debug(LD_CIRC,"Hunting for a circ to cannibalize: purpose %d, uptime %d, capacity %d, internal %d", purpose, need_uptime, need_capacity, internal);
 
   for (circ=global_circuitlist; circ; circ = circ->next) {
     if (CIRCUIT_IS_ORIGIN(circ) &&
@@ -545,8 +578,19 @@ circuit_get_clean_open(uint8_t purpose, int need_uptime,
         (!need_uptime || circ->build_state->need_uptime) &&
         (!need_capacity || circ->build_state->need_capacity) &&
         (!internal || circ->build_state->is_internal)) {
+      if (info) {
+        /* need to make sure we don't duplicate hops */
+        crypt_path_t *hop = circ->cpath;
+        do {
+          if (!memcmp(hop->extend_info->identity_digest,
+                      info->identity_digest, DIGEST_LEN))
+            goto next;
+          hop=hop->next;
+        } while (hop!=circ->cpath);
+      }
       if (!best || (best->build_state->need_uptime && !need_uptime))
         best = circ;
+      next: ;
     }
   }
   return best;
@@ -613,7 +657,8 @@ _circuit_mark_for_close(circuit_t *circ, int line, const char *file)
   tor_assert(file);
 
   if (circ->marked_for_close) {
-    log(LOG_WARN,"Duplicate call to circuit_mark_for_close at %s:%d"
+    log(LOG_WARN,LD_BUG,
+        "Duplicate call to circuit_mark_for_close at %s:%d"
         " (first at %s:%d)", file, line,
         circ->marked_for_close_file, circ->marked_for_close);
     return;
@@ -640,7 +685,7 @@ _circuit_mark_for_close(circuit_t *circ, int line, const char *file)
     tor_assert(circ->state == CIRCUIT_STATE_OPEN);
     tor_assert(circ->build_state->chosen_exit);
     /* treat this like getting a nack from it */
-    log_fn(LOG_INFO,"Failed intro circ %s to %s (awaiting ack). Removing from descriptor.",
+    info(LD_REND,"Failed intro circ %s to %s (awaiting ack). Removing from descriptor.",
            safe_str(circ->rend_query),
            safe_str(build_state_get_exit_nickname(circ->build_state)));
     rend_client_remove_intro_point(circ->build_state->chosen_exit,
@@ -701,7 +746,7 @@ assert_cpath_layer_ok(const crypt_path_t *cp)
       /* tor_assert(cp->dh_handshake_state); */
       break;
     default:
-      log_fn(LOG_ERR,"Unexpected state %d",cp->state);
+      log_fn(LOG_ERR, LD_BUG, "Unexpected state %d", cp->state);
       tor_assert(0);
     }
   tor_assert(cp->package_window >= 0);
@@ -763,6 +808,7 @@ assert_circuit_ok(const circuit_t *c)
   tor_assert(c->deliver_window >= 0);
   tor_assert(c->package_window >= 0);
   if (c->state == CIRCUIT_STATE_OPEN) {
+    tor_assert(!c->onionskin);
     if (c->cpath) {
       tor_assert(CIRCUIT_IS_ORIGIN(c));
       tor_assert(!c->n_crypto);
