@@ -1,7 +1,7 @@
 /* Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: control.c,v 1.138 2005/10/05 02:09:27 arma Exp $ */
-const char control_c_id[] = "$Id: control.c,v 1.138 2005/10/05 02:09:27 arma Exp $";
+/* $Id: control.c,v 1.147 2005/11/05 20:15:26 nickm Exp $ */
+const char control_c_id[] = "$Id: control.c,v 1.147 2005/11/05 20:15:26 nickm Exp $";
 
 /**
  * \file control.c
@@ -431,12 +431,15 @@ get_escaped_string(const char *start, size_t in_len_max,
 /** Acts like sprintf, but writes its formatted string to the end of
  * <b>conn</b>-\>outbuf.  The message may be truncated if it is too long,
  * but it will always end with a CRLF sequence.
- */
+ *
+ * Currently the length of the message is limited to 1024 (including the
+ * ending \n\r\0. */
 static void
 connection_printf_to_buf(connection_t *conn, const char *format, ...)
 {
+#define CONNECTION_PRINTF_TO_BUF_BUFFERSIZE 1024
   va_list ap;
-  char buf[1024];
+  char buf[CONNECTION_PRINTF_TO_BUF_BUFFERSIZE];
   int r;
   size_t len;
   va_start(ap,format);
@@ -444,9 +447,9 @@ connection_printf_to_buf(connection_t *conn, const char *format, ...)
   va_end(ap);
   len = strlen(buf);
   if (memcmp("\r\n\0", buf+len-2, 3)) {
-    buf[1023] = '\0';
-    buf[1022] = '\n';
-    buf[1021] = '\r';
+    buf[CONNECTION_PRINTF_TO_BUF_BUFFERSIZE-1] = '\0';
+    buf[CONNECTION_PRINTF_TO_BUF_BUFFERSIZE-2] = '\n';
+    buf[CONNECTION_PRINTF_TO_BUF_BUFFERSIZE-3] = '\r';
   }
   connection_write_to_buf(buf, len, conn);
 }
@@ -559,27 +562,12 @@ send_control0_event(uint16_t event, uint32_t len, const char *body)
 }
 
 /* Send an event to all v1 controllers that are listening for code
- * <b>event</b>.  The event's body is created by the printf-style format in
- * <b>format</b>, and other arguments as provided. */
+ * <b>event</b>.  The event's body is given by <b>msg</b>. */
 static void
-send_control1_event(uint16_t event, const char *format, ...)
+send_control1_event_string(uint16_t event, const char *msg)
 {
   connection_t **conns;
-  int n_conns, i, r;
-  char buf[1024]; /* XXXX Length */
-  va_list ap;
-  size_t len;
-
-  va_start(ap, format);
-  r = tor_vsnprintf(buf, sizeof(buf), format, ap);
-  va_end(ap);
-
-  len = strlen(buf);
-  if (memcmp("\r\n\0", buf+len-2, 3)) {
-    buf[1023] = '\0';
-    buf[1022] = '\n';
-    buf[1021] = '\r';
-  }
+  int n_conns, i;
 
   tor_assert(event >= _EVENT_MIN && event <= _EVENT_MAX);
 
@@ -589,13 +577,43 @@ send_control1_event(uint16_t event, const char *format, ...)
         !conns[i]->marked_for_close &&
         conns[i]->state == CONTROL_CONN_STATE_OPEN_V1 &&
         conns[i]->event_mask & (1<<event)) {
-      connection_write_to_buf(buf, len, conns[i]);
+      connection_write_to_buf(msg, strlen(msg), conns[i]);
       if (event == EVENT_ERR_MSG)
         _connection_controller_force_write(conns[i]);
     }
   }
 }
 
+/* Send an event to all v1 controllers that are listening for code
+ * <b>event</b>.  The event's body is created by the printf-style format in
+ * <b>format</b>, and other arguments as provided.
+ *
+ * Currently the length of the message is limited to 1024 (including the
+ * ending \n\r\0. */
+static void
+send_control1_event(uint16_t event, const char *format, ...)
+{
+#define SEND_CONTROL1_EVENT_BUFFERSIZE 1024
+  int r;
+  char buf[SEND_CONTROL1_EVENT_BUFFERSIZE]; /* XXXX Length */
+  va_list ap;
+  size_t len;
+
+  va_start(ap, format);
+  r = tor_vsnprintf(buf, sizeof(buf), format, ap);
+  va_end(ap);
+
+  len = strlen(buf);
+  if (memcmp("\r\n\0", buf+len-2, 3)) {
+    /* if it is not properly terminated, do it now */
+    buf[SEND_CONTROL1_EVENT_BUFFERSIZE-1] = '\0';
+    buf[SEND_CONTROL1_EVENT_BUFFERSIZE-2] = '\n';
+    buf[SEND_CONTROL1_EVENT_BUFFERSIZE-3] = '\r';
+  }
+
+  send_control1_event_string(event, buf);
+}
+
 /** Given a text circuit <b>id</b>, return the corresponding circuit. */
 static circuit_t *
 get_circ(const char *id)
@@ -673,7 +691,7 @@ control_setconf_helper(connection_t *conn, uint32_t len, char *body,
     *outp = '\0';
 
     if (config_get_lines(config, &lines) < 0) {
-      log_fn(LOG_WARN,"Controller gave us config lines we can't parse.");
+      warn(LD_CONTROL,"Controller gave us config lines we can't parse.");
       connection_write_str_to_buf("551 Couldn't parse configuration\r\n", conn);
       tor_free(config);
       return 0;
@@ -681,7 +699,7 @@ control_setconf_helper(connection_t *conn, uint32_t len, char *body,
     tor_free(config);
   } else {
     if (config_get_lines(body, &lines) < 0) {
-      log_fn(LOG_WARN,"Controller gave us config lines we can't parse.");
+      warn(LD_CONTROL,"Controller gave us config lines we can't parse.");
       send_control0_error(conn, ERR_SYNTAX, "Couldn't parse configuration");
       return 0;
     }
@@ -690,7 +708,7 @@ control_setconf_helper(connection_t *conn, uint32_t len, char *body,
   if ((r=options_trial_assign(lines, use_defaults, clear_first)) < 0) {
     int v0_err;
     const char *msg;
-    log_fn(LOG_WARN,"Controller gave us config lines that didn't validate.");
+    warn(LD_CONTROL,"Controller gave us config lines that didn't validate.");
     switch (r) {
       case -1:
         v0_err = ERR_UNRECOGNIZED_CONFIG_KEY;
@@ -828,10 +846,14 @@ handle_control_getconf(connection_t *conn, uint32_t body_len, const char *body)
   }
 
  done:
-  if (answers) SMARTLIST_FOREACH(answers, char *, cp, tor_free(cp));
-  if (questions) SMARTLIST_FOREACH(questions, char *, cp, tor_free(cp));
-  smartlist_free(answers);
-  smartlist_free(questions);
+  if (answers) {
+    SMARTLIST_FOREACH(answers, char *, cp, tor_free(cp));
+    smartlist_free(answers);
+  }
+  if (questions) {
+    SMARTLIST_FOREACH(questions, char *, cp, tor_free(cp));
+    smartlist_free(questions);
+  }
   smartlist_free(unrecognized);
   tor_free(msg);
 
@@ -845,6 +867,7 @@ handle_control_setevents(connection_t *conn, uint32_t len, const char *body)
 {
   uint16_t event_code;
   uint32_t event_mask = 0;
+  unsigned int extended = 0;
 
   if (STATE_IS_V0(conn->state)) {
     if (len % 2) {
@@ -868,7 +891,10 @@ handle_control_setevents(connection_t *conn, uint32_t len, const char *body)
                            SPLIT_SKIP_SPACE|SPLIT_IGNORE_BLANK, 0);
     SMARTLIST_FOREACH(events, const char *, ev,
       {
-        if (!strcasecmp(ev, "CIRC"))
+        if (!strcasecmp(ev, "EXTENDED")) {
+          extended = 1;
+          continue;
+        } else if (!strcasecmp(ev, "CIRC"))
           event_code = EVENT_CIRCUIT_STATUS;
         else if (!strcasecmp(ev, "STREAM"))
           event_code = EVENT_STREAM_STATUS;
@@ -903,6 +929,7 @@ handle_control_setevents(connection_t *conn, uint32_t len, const char *body)
     smartlist_free(events);
   }
   conn->event_mask = event_mask;
+  conn->control_events_are_extended = extended;
 
   control_update_global_event_mask();
   send_control_done(conn);
@@ -941,6 +968,7 @@ decode_hashed_password(char *buf, const char *hashed)
 static int
 handle_control_authenticate(connection_t *conn, uint32_t len, const char *body)
 {
+  int used_quoted_string = 0;
   or_options_t *options = get_options();
   char *password;
   size_t password_len;
@@ -954,7 +982,7 @@ handle_control_authenticate(connection_t *conn, uint32_t len, const char *body)
         ++i;
       password = tor_malloc(i/2 + 1);
       if (base16_decode(password, i/2+1, body, i)<0) {
-        connection_write_str_to_buf("551 Invalid hexadecimal encoding\r\n", conn);
+        connection_write_str_to_buf("551 Invalid hexadecimal encoding.  Maybe you tried a plain text password?  If so, the standard requires you put it in double quotes.\r\n", conn);
         tor_free(password);
         return 0;
       }
@@ -964,9 +992,10 @@ handle_control_authenticate(connection_t *conn, uint32_t len, const char *body)
       password_len = 0;
     } else {
       if (!get_escaped_string(body, len, &password, &password_len)) {
-        connection_write_str_to_buf("551 Invalid quoted string\r\n", conn);
+        connection_write_str_to_buf("551 Invalid quoted string.  You need to put the password in double quotes.\r\n", conn);
         return 0;
       }
+      used_quoted_string = 1;
     }
   }
   if (options->CookieAuthentication) {
@@ -978,7 +1007,7 @@ handle_control_authenticate(connection_t *conn, uint32_t len, const char *body)
     char expected[S2K_SPECIFIER_LEN+DIGEST_LEN];
     char received[DIGEST_LEN];
     if (decode_hashed_password(expected, options->HashedControlPassword)<0) {
-      log_fn(LOG_WARN,"Couldn't decode HashedControlPassword: invalid base64");
+      warn(LD_CONTROL,"Couldn't decode HashedControlPassword: invalid base16");
       goto err;
     }
     secret_to_key(received,DIGEST_LEN,password,password_len,expected);
@@ -996,11 +1025,14 @@ handle_control_authenticate(connection_t *conn, uint32_t len, const char *body)
     send_control0_error(conn,ERR_REJECTED_AUTHENTICATION,"Authentication failed");
   else {
     tor_free(password);
-    connection_write_str_to_buf("515 Authentication failed\r\n", conn);
+    if (used_quoted_string)
+      connection_write_str_to_buf("515 Authentication failed\r\n", conn);
+    else
+      connection_write_str_to_buf("515 Authentication failed.  Maybe you tried a plain text password?  If so, the standard requires you put it in double quotes.\r\n", conn);
   }
   return 0;
  ok:
-  log_fn(LOG_INFO, "Authenticated control connection (%d)", conn->s);
+  info(LD_CONTROL, "Authenticated control connection (%d)", conn->s);
   send_control_done(conn);
   if (STATE_IS_V0(conn->state))
     conn->state = CONTROL_CONN_STATE_OPEN_V0;
@@ -1116,17 +1148,17 @@ handle_control_mapaddress(connection_t *conn, uint32_t len, const char *body)
       const char *from = smartlist_get(elts,0);
       const char *to = smartlist_get(elts,1);
       if (!is_plausible_address(from)) {
-        log_fn(LOG_WARN,"Skipping invalid argument '%s' in MapAddress msg",from);
+        warn(LD_CONTROL,"Skipping invalid argument '%s' in MapAddress msg",from);
       } else if (!is_plausible_address(to)) {
-        log_fn(LOG_WARN,"Skipping invalid argument '%s' in MapAddress msg",to);
+        warn(LD_CONTROL,"Skipping invalid argument '%s' in MapAddress msg",to);
       } else if (!strcmp(from, ".") || !strcmp(from, "0.0.0.0")) {
         const char *address = addressmap_register_virtual_address(
               !strcmp(from,".") ? RESOLVED_TYPE_HOSTNAME : RESOLVED_TYPE_IPV4,
                tor_strdup(to));
         if (!address) {
-          log_fn(LOG_WARN,
-                 "Unable to allocate address for '%s' in MapAddress msg",
-                 safe_str(line));
+          warn(LD_CONTROL,
+               "Unable to allocate address for '%s' in MapAddress msg",
+               safe_str(line));
         } else {
           size_t anslen = strlen(address)+strlen(to)+8;
           char *ans = tor_malloc(anslen);
@@ -1148,7 +1180,7 @@ handle_control_mapaddress(connection_t *conn, uint32_t len, const char *body)
         }
       }
     } else {
-      log_fn(LOG_WARN, "Skipping MapAddress line with wrong number of items.");
+      warn(LD_CONTROL, "Skipping MapAddress line with wrong number of items.");
     }
     SMARTLIST_FOREACH(elts, char *, cp, tor_free(cp));
     smartlist_clear(elts);
@@ -1223,18 +1255,17 @@ handle_getinfo_helper(const char *question, char **answer)
     *answer = list_getinfo_options();
   } else if (!strcmpstart(question, "desc/id/")) {
     routerinfo_t *ri = router_get_by_hexdigest(question+strlen("desc/id/"));
-    if (ri && ri->signed_descriptor)
-      *answer = tor_strdup(ri->signed_descriptor);
+    if (ri && ri->cache_info.signed_descriptor)
+      *answer = tor_strdup(ri->cache_info.signed_descriptor);
   } else if (!strcmpstart(question, "desc/name/")) {
     routerinfo_t *ri = router_get_by_nickname(question+strlen("desc/name/"),1);
-    if (ri && ri->signed_descriptor)
-      *answer = tor_strdup(ri->signed_descriptor);
+    if (ri && ri->cache_info.signed_descriptor)
+      *answer = tor_strdup(ri->cache_info.signed_descriptor);
   } else if (!strcmpstart(question, "unregistered-servers-")) {
     *answer = dirserver_getinfo_unregistered(question +
                                              strlen("unregistered-servers-"));
   } else if (!strcmp(question, "network-status")) {
-    routerlist_t *routerlist;
-    router_get_routerlist(&routerlist);
+    routerlist_t *routerlist = router_get_routerlist();
     if (!routerlist || !routerlist->routers ||
         list_server_status(routerlist->routers, answer) < 0) {
       return -1;
@@ -1299,7 +1330,7 @@ handle_getinfo_helper(const char *question, char **answer)
         case AP_CONN_STATE_OPEN:
           state = "SUCCEEDED"; break;
         default:
-          log_fn(LOG_WARN, "Asked for stream in unknown state %d",
+          warn(LD_BUG, "Asked for stream in unknown state %d",
                  conns[i]->state);
           continue;
         }
@@ -1444,10 +1475,14 @@ handle_control_getinfo(connection_t *conn, uint32_t len, const char *body)
   }
 
  done:
-  if (answers) SMARTLIST_FOREACH(answers, char *, cp, tor_free(cp));
-  if (questions) SMARTLIST_FOREACH(questions, char *, cp, tor_free(cp));
-  smartlist_free(answers);
-  smartlist_free(questions);
+  if (answers) {
+    SMARTLIST_FOREACH(answers, char *, cp, tor_free(cp));
+    smartlist_free(answers);
+  }
+  if (questions) {
+    SMARTLIST_FOREACH(questions, char *, cp, tor_free(cp));
+    smartlist_free(questions);
+  }
   smartlist_free(unrecognized);
   tor_free(msg);
 
@@ -1558,7 +1593,7 @@ handle_control_extendcircuit(connection_t *conn, uint32_t len,
     if (circ->state == CIRCUIT_STATE_OPEN) {
       circ->state = CIRCUIT_STATE_BUILDING;
       if (circuit_send_next_onion_skin(circ) < 0) {
-        log_fn(LOG_INFO,"send_next_onion_skin failed; circuit marked for closing.");
+        info(LD_CONTROL,"send_next_onion_skin failed; circuit marked for closing.");
         circuit_mark_for_close(circ);
         if (v0)
           send_control0_error(conn, ERR_INTERNAL, "couldn't send onion skin");
@@ -1868,8 +1903,8 @@ handle_control_closecircuit(connection_t *conn, uint32_t len,
         if (!strcasecmp(smartlist_get(args, i), "IfUnused"))
           safe = 1;
         else
-          log_fn(LOG_INFO, "Skipping unknown option %s",
-                 (char*)smartlist_get(args,i));
+          info(LD_CONTROL, "Skipping unknown option %s",
+               (char*)smartlist_get(args,i));
       }
     }
     SMARTLIST_FOREACH(args, char *, cp, tor_free(cp));
@@ -1897,7 +1932,7 @@ handle_control_fragments(connection_t *conn, uint16_t command_type,
 {
   if (command_type == CONTROL0_CMD_FRAGMENTHEADER) {
     if (conn->incoming_cmd) {
-      log_fn(LOG_WARN, "Dropping incomplete fragmented command");
+      warn(LD_CONTROL, "Dropping incomplete fragmented command");
       tor_free(conn->incoming_cmd);
     }
     if (body_len < 6) {
@@ -1949,7 +1984,7 @@ connection_control_reached_eof(connection_t *conn)
   tor_assert(conn);
   tor_assert(conn->type == CONN_TYPE_CONTROL);
 
-  log_fn(LOG_INFO,"Control connection reached EOF. Closing.");
+  info(LD_CONTROL,"Control connection reached EOF. Closing.");
   connection_mark_for_close(conn);
   return 0;
 }
@@ -2022,10 +2057,6 @@ connection_control_process_inbuf_v1(connection_t *conn)
          && !TOR_ISSPACE(conn->incoming_cmd[cmd_len]))
     ++cmd_len;
 
-  /*
-  log_fn(LOG_NOTICE, "READ A COMMAND FROM THE BUFFER: <%s> [%d]",
-         conn->incoming_cmd, (int)cmd_len);
-  */
   data_len -= cmd_len;
   conn->incoming_cmd[cmd_len]='\0';
   args = conn->incoming_cmd+cmd_len+1;
@@ -2033,10 +2064,6 @@ connection_control_process_inbuf_v1(connection_t *conn)
     ++args;
     --data_len;
   }
-  /*
-  log_fn(LOG_NOTICE, "COMMAND IS: <%s>", conn->incoming_cmd);
-  log_fn(LOG_NOTICE, "ARGS ARE: <%s>", args);
-  */
 
   if (!strcasecmp(conn->incoming_cmd, "QUIT")) {
     connection_write_str_to_buf("250 closing connection\r\n", conn);
@@ -2122,13 +2149,13 @@ connection_control_process_inbuf_v0(connection_t *conn)
     {
     case -2:
       tor_free(body);
-      log_fn(LOG_INFO, "Detected v1 control protocol on connection (fd %d)",
-             conn->s);
+      info(LD_CONTROL, "Detected v1 control protocol on connection (fd %d)",
+           conn->s);
       conn->state = CONTROL_CONN_STATE_NEEDAUTH_V1;
       return connection_control_process_inbuf_v1(conn);
     case -1:
       tor_free(body);
-      log_fn(LOG_WARN, "Error in control command. Failing.");
+      warn(LD_CONTROL, "Error in control command. Failing.");
       return -1;
     case 0:
       /* Control command not all here yet. Wait. */
@@ -2144,8 +2171,8 @@ connection_control_process_inbuf_v0(connection_t *conn)
    * commands will be considered. */
   if (conn->state == CONTROL_CONN_STATE_NEEDAUTH_V0 &&
       command_type != CONTROL0_CMD_AUTHENTICATE) {
-    log_fn(LOG_WARN, "Rejecting '%s' command; authentication needed.",
-           control_cmd_to_string(command_type));
+    info(LD_CONTROL, "Rejecting '%s' command; authentication needed.",
+         control_cmd_to_string(command_type));
     send_control0_error(conn, ERR_UNAUTHORIZED, "Authentication required");
     tor_free(body);
     goto again;
@@ -2164,7 +2191,7 @@ connection_control_process_inbuf_v0(connection_t *conn)
     body = conn->incoming_cmd;
     conn->incoming_cmd = NULL;
   } else if (conn->incoming_cmd) {
-    log_fn(LOG_WARN, "Dropping incomplete fragmented command");
+    warn(LD_CONTROL, "Dropping incomplete fragmented command");
     tor_free(conn->incoming_cmd);
   }
 
@@ -2232,17 +2259,17 @@ connection_control_process_inbuf_v0(connection_t *conn)
     case CONTROL0_CMD_CONFVALUE:
     case CONTROL0_CMD_EVENT:
     case CONTROL0_CMD_INFOVALUE:
-      log_fn(LOG_WARN, "Received client-only '%s' command; ignoring.",
+      warn(LD_CONTROL, "Received client-only '%s' command; ignoring.",
              control_cmd_to_string(command_type));
       send_control0_error(conn, ERR_UNRECOGNIZED_TYPE,
                          "Command type only valid from server to tor client");
       break;
     case CONTROL0_CMD_FRAGMENTHEADER:
     case CONTROL0_CMD_FRAGMENT:
-      log_fn(LOG_WARN, "Recieved command fragment out of order; ignoring.");
+      warn(LD_CONTROL, "Recieved command fragment out of order; ignoring.");
       send_control0_error(conn, ERR_SYNTAX, "Bad fragmentation on command.");
     default:
-      log_fn(LOG_WARN, "Received unrecognized command type %d; ignoring.",
+      warn(LD_CONTROL, "Received unrecognized command type %d; ignoring.",
              (int)command_type);
       send_control0_error(conn, ERR_UNRECOGNIZED_TYPE,
                          "Unrecognized command type");
@@ -2298,7 +2325,7 @@ control_event_circuit_status(circuit_t *circ, circuit_status_event_t tp)
       case CIRC_EVENT_FAILED: status = "FAILED"; break;
       case CIRC_EVENT_CLOSED: status = "CLOSED"; break;
       default:
-        log_fn(LOG_WARN, "Unrecognized status code %d", (int)tp);
+        warn(LD_BUG, "Unrecognized status code %d", (int)tp);
         return 0;
       }
     send_control1_event(EVENT_CIRCUIT_STATUS,
@@ -2369,7 +2396,7 @@ control_event_stream_status(connection_t *conn, stream_status_event_t tp)
       case STREAM_EVENT_NEW_RESOLVE: status = "NEWRESOLVE"; break;
       case STREAM_EVENT_FAILED_RETRIABLE: status = "DETACHED"; break;
       default:
-        log_fn(LOG_WARN, "Unrecognized status code %d", (int)tp);
+        warn(LD_BUG, "Unrecognized status code %d", (int)tp);
         return 0;
       }
     circ = circuit_get_by_edge_conn(conn);
@@ -2411,7 +2438,7 @@ control_event_or_conn_status(connection_t *conn,or_conn_status_event_t tp)
       case OR_CONN_EVENT_FAILED: status = "FAILED"; break;
       case OR_CONN_EVENT_CLOSED: status = "CLOSED"; break;
       default:
-        log_fn(LOG_WARN, "Unrecognized status code %d", (int)tp);
+        warn(LD_BUG, "Unrecognized status code %d", (int)tp);
         return 0;
       }
     send_control1_event(EVENT_OR_CONN_STATUS,
@@ -2463,7 +2490,7 @@ enable_control_logging(void)
 
 /** We got a log message: tell any interested control connections. */
 void
-control_event_logmsg(int severity, const char *msg)
+control_event_logmsg(int severity, unsigned int domain, const char *msg)
 {
   int oldlog, event;
 
@@ -2530,7 +2557,7 @@ control_event_descriptors_changed(smartlist_t *routers)
   identities = smartlist_create();
   SMARTLIST_FOREACH(routers, routerinfo_t *, r,
   {
-    base16_encode(buf,sizeof(buf),r->identity_digest,DIGEST_LEN);
+    base16_encode(buf,sizeof(buf),r->cache_info.identity_digest,DIGEST_LEN);
     smartlist_add(identities, tor_strdup(buf));
   });
   if (EVENT_IS_INTERESTING0(EVENT_NEW_DESC)) {
@@ -2588,7 +2615,7 @@ init_cookie_authentication(int enabled)
   authentication_cookie_is_set = 1;
   if (write_bytes_to_file(fname, authentication_cookie,
                           AUTHENTICATION_COOKIE_LEN, 1)) {
-    log_fn(LOG_WARN,"Error writing authentication cookie.");
+    warn(LD_FS,"Error writing authentication cookie.");
     return -1;
   }
 
