@@ -1,8 +1,8 @@
 /* Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: dirserv.c,v 1.246 2005/10/06 04:33:40 nickm Exp $ */
-const char dirserv_c_id[] = "$Id: dirserv.c,v 1.246 2005/10/06 04:33:40 nickm Exp $";
+/* $Id: dirserv.c,v 1.266 2005/11/05 20:15:27 nickm Exp $ */
+const char dirserv_c_id[] = "$Id: dirserv.c,v 1.266 2005/11/05 20:15:27 nickm Exp $";
 
 #include "or.h"
 
@@ -135,7 +135,7 @@ dirserv_add_own_fingerprint(const char *nickname, crypto_pk_env_t *pk)
 {
   char fp[FINGERPRINT_LEN+1];
   if (crypto_pk_get_fingerprint(pk, fp, 0)<0) {
-    log_fn(LOG_ERR, "Error computing fingerprint");
+    err(LD_BUG, "Error computing fingerprint");
     return -1;
   }
   if (!fingerprint_list)
@@ -161,13 +161,13 @@ dirserv_parse_fingerprint_file(const char *fname)
 
   cf = read_file_to_str(fname, 0);
   if (!cf) {
-    log_fn(LOG_WARN, "Cannot open fingerprint file %s", fname);
+    warn(LD_FS, "Cannot open fingerprint file %s", fname);
     return -1;
   }
   result = config_get_lines(cf, &front);
   tor_free(cf);
   if (result < 0) {
-    log_fn(LOG_WARN, "Error reading from fingerprint file");
+    warn(LD_CONFIG, "Error reading from fingerprint file");
     return -1;
   }
 
@@ -176,31 +176,34 @@ dirserv_parse_fingerprint_file(const char *fname)
   for (list=front; list; list=list->next) {
     nickname = list->key; fingerprint = list->value;
     if (strlen(nickname) > MAX_NICKNAME_LEN) {
-      log(LOG_NOTICE, "Nickname '%s' too long in fingerprint file. Skipping.", nickname);
+      notice(LD_CONFIG,
+            "Nickname '%s' too long in fingerprint file. Skipping.", nickname);
       continue;
     }
     if (!is_legal_nickname(nickname) &&
         strcasecmp(nickname, "!reject") &&
         strcasecmp(nickname, "!invalid")) {
-      log(LOG_NOTICE, "Invalid nickname '%s' in fingerprint file. Skipping.", nickname);
+      notice(LD_CONFIG,
+             "Invalid nickname '%s' in fingerprint file. Skipping.", nickname);
       continue;
     }
     if (strlen(fingerprint) != FINGERPRINT_LEN ||
         !crypto_pk_check_fingerprint_syntax(fingerprint)) {
-      log_fn(LOG_NOTICE, "Invalid fingerprint (nickname '%s', fingerprint %s). Skipping.",
+      notice(LD_CONFIG,
+             "Invalid fingerprint (nickname '%s', fingerprint %s). Skipping.",
              nickname, fingerprint);
       continue;
     }
     if (0==strcasecmp(nickname, DEFAULT_CLIENT_NICKNAME)) {
       /* If you approved an OR called "client", then clients who use
        * the default nickname could all be rejected.  That's no good. */
-      log(LOG_NOTICE,
+      notice(LD_CONFIG,
           "Authorizing a nickname '%s' would break many clients; skipping.",
           DEFAULT_CLIENT_NICKNAME);
       continue;
     }
     if (add_fingerprint_to_dir(nickname, fingerprint, fingerprint_list_new) != 0)
-      log(LOG_NOTICE, "Duplicate nickname '%s'.", nickname);
+      notice(LD_CONFIG, "Duplicate nickname '%s'.", nickname);
   }
 
   config_free_lines(front);
@@ -228,11 +231,11 @@ dirserv_router_get_status(const routerinfo_t *router, const char **msg)
     fingerprint_list = smartlist_create();
 
   if (crypto_pk_get_fingerprint(router->identity_pkey, fp, 0)) {
-    log_fn(LOG_WARN,"Error computing fingerprint");
+    warn(LD_BUG,"Error computing fingerprint");
     return -1;
   }
 
-  log_fn(LOG_DEBUG, "%d fingerprints known.", smartlist_len(fingerprint_list));
+  debug(LD_DIRSERV, "%d fingerprints known.", smartlist_len(fingerprint_list));
   SMARTLIST_FOREACH(fingerprint_list, fingerprint_entry_t *, ent,
   {
     if (!strcasecmp(fp,ent->fingerprint))
@@ -260,32 +263,32 @@ dirserv_router_get_status(const routerinfo_t *router, const char **msg)
                        router->addr, router->or_port, authdir_invalid_policy);
 
     if (rej == ADDR_POLICY_PROBABLY_REJECTED || rej == ADDR_POLICY_REJECTED) {
-      log_fn(LOG_INFO, "Rejecting '%s' because of address %s",
-             router->nickname, router->address);
+      info(LD_DIRSERV, "Rejecting '%s' because of address %s",
+           router->nickname, router->address);
       if (msg)
         *msg = "Authdir is rejecting routers in this range.";
       return FP_REJECT;
     }
     if (inv == ADDR_POLICY_PROBABLY_REJECTED || inv == ADDR_POLICY_REJECTED) {
-      log_fn(LOG_INFO, "Not marking '%s' valid because of address %s",
-             router->nickname, router->address);
+      info(LD_DIRSERV, "Not marking '%s' valid because of address %s",
+           router->nickname, router->address);
       return FP_INVALID;
     }
     if (tor_version_as_new_as(router->platform,"0.1.0.2-rc"))
       return FP_VALID;
     else
       return FP_INVALID;
-    log_fn(LOG_INFO,"No fingerprint found for '%s'",router->nickname);
+    info(LD_DIRSERV,"No fingerprint found for '%s'",router->nickname);
     return 0;
   }
   if (0==strcasecmp(nn_ent->fingerprint, fp)) {
-    log_fn(LOG_DEBUG,"Good fingerprint for '%s'",router->nickname);
+    debug(LD_DIRSERV,"Good fingerprint for '%s'",router->nickname);
     return FP_NAMED; /* Right fingerprint. */
   } else {
-    log_fn(LOG_WARN,"Mismatched fingerprint for '%s': expected '%s' got '%s'. ContactInfo '%s', platform '%s'.)",
-           router->nickname, nn_ent->fingerprint, fp,
-           router->contact_info ? router->contact_info : "",
-           router->platform ? router->platform : "");
+    warn(LD_DIRSERV,"Mismatched fingerprint for '%s': expected '%s' got '%s'. ContactInfo '%s', platform '%s'.)",
+         router->nickname, nn_ent->fingerprint, fp,
+         router->contact_info ? router->contact_info : "",
+         router->platform ? router->platform : "");
     if (msg)
       *msg = "Rejected: There is already a verified server with this nickname and a different fingerprint.";
     return FP_REJECT; /* Wrong fingerprint. */
@@ -333,16 +336,6 @@ dirserv_free_fingerprint_list(void)
  *    Descriptor list
  */
 
-static smartlist_t *
-get_descriptor_list(void)
-{
-  routerlist_t *routerlist;
-  router_get_routerlist(&routerlist);
-  if (!routerlist)
-    return NULL;
-  return routerlist->routers;
-}
-
 /** Return -1 if <b>ri</b> has a private or otherwise bad address,
  * unless we're configured to not care. Return 0 if all ok. */
 static int
@@ -352,13 +345,14 @@ dirserv_router_has_valid_address(routerinfo_t *ri)
   if (get_options()->DirAllowPrivateAddresses)
     return 0; /* whatever it is, we're fine with it */
   if (!tor_inet_aton(ri->address, &iaddr)) {
-    log_fn(LOG_INFO,"Router '%s' published non-IP address '%s'. Refusing.",
-           ri->nickname, ri->address);
+    info(LD_DIRSERV,"Router '%s' published non-IP address '%s'. Refusing.",
+         ri->nickname, ri->address);
     return -1;
   }
   if (is_internal_IP(ntohl(iaddr.s_addr))) {
-    log_fn(LOG_INFO,"Router '%s' published internal IP address '%s'. Refusing.",
-           ri->nickname, ri->address);
+    info(LD_DIRSERV,
+         "Router '%s' published internal IP address '%s'. Refusing.",
+         ri->nickname, ri->address);
     return -1; /* it's a private IP, we should reject it */
   }
   return 0;
@@ -382,24 +376,24 @@ authdir_wants_to_reject_router(routerinfo_t *ri,
 
   /* Is there too much clock skew? */
   now = time(NULL);
-  if (ri->published_on > now+ROUTER_ALLOW_SKEW) {
-    log_fn(LOG_NOTICE, "Publication time for nickname '%s' is too far (%d minutes) in the future; possible clock skew. Not adding (ContactInfo '%s', platform '%s').",
-           ri->nickname, (int)((ri->published_on-now)/60),
+  if (ri->cache_info.published_on > now+ROUTER_ALLOW_SKEW) {
+    notice(LD_DIRSERV, "Publication time for nickname '%s' is too far (%d minutes) in the future; possible clock skew. Not adding (ContactInfo '%s', platform '%s').",
+           ri->nickname, (int)((ri->cache_info.published_on-now)/60),
            ri->contact_info ? ri->contact_info : "",
            ri->platform ? ri->platform : "");
     *msg = "Rejected: Your clock is set too far in the future, or your timezone is not correct.";
     return -1;
   }
-  if (ri->published_on < now-ROUTER_MAX_AGE) {
-    log_fn(LOG_NOTICE, "Publication time for router with nickname '%s' is too far (%d minutes) in the past. Not adding (ContactInfo '%s', platform '%s').",
-           ri->nickname, (int)((now-ri->published_on)/60),
+  if (ri->cache_info.published_on < now-ROUTER_MAX_AGE) {
+    notice(LD_DIRSERV, "Publication time for router with nickname '%s' is too far (%d minutes) in the past. Not adding (ContactInfo '%s', platform '%s').",
+           ri->nickname, (int)((now-ri->cache_info.published_on)/60),
            ri->contact_info ? ri->contact_info : "",
            ri->platform ? ri->platform : "");
     *msg = "Rejected: Server is expired, or your clock is too far in the past, or your timezone is not correct.";
     return -1;
   }
   if (dirserv_router_has_valid_address(ri) < 0) {
-    log_fn(LOG_NOTICE, "Router with nickname '%s' has invalid address '%s'. Not adding (ContactInfo '%s', platform '%s').",
+    notice(LD_DIRSERV, "Router with nickname '%s' has invalid address '%s'. Not adding (ContactInfo '%s', platform '%s').",
            ri->nickname, ri->address,
            ri->contact_info ? ri->contact_info : "",
            ri->platform ? ri->platform : "");
@@ -439,17 +433,31 @@ int
 dirserv_add_descriptor(const char *desc, const char **msg)
 {
   int r;
-  routerinfo_t *ri = NULL;
+  routerinfo_t *ri = NULL, *ri_old = NULL;
   tor_assert(msg);
   *msg = NULL;
 
   /* Check: is the descriptor syntactically valid? */
   ri = router_parse_entry_from_string(desc, NULL);
   if (!ri) {
-    log(LOG_WARN, "Couldn't parse descriptor");
+    warn(LD_DIRSERV, "Couldn't parse uploaded server descriptor");
     *msg = "Rejected: Couldn't parse server descriptor.";
     return -2;
   }
+  /* Check whether this descriptor is semantically identical to the last one
+   * from this server.  (We do this here and not in router_add_to_routerlist
+   * because we want to be able to accept the newest router descriptor that
+   * another authority has, so we all converge on the same one.) */
+  ri_old = router_get_by_digest(ri->cache_info.identity_digest);
+  if (ri_old && ri_old->cache_info.published_on < ri->cache_info.published_on
+      && router_differences_are_cosmetic(ri_old, ri)) {
+    info(LD_DIRSERV,
+         "Not replacing descriptor from '%s'; differences are cosmetic.",
+         ri->nickname);
+    *msg = "Not replacing router descriptor; no information has changed since the last one with this identity.";
+    routerinfo_free(ri);
+    return 0;
+  }
   if ((r = router_add_to_routerlist(ri, msg, 0))<0) {
     return r == -1 ? 0 : -1;
   } else {
@@ -474,33 +482,30 @@ directory_remove_invalid(void)
 {
   int i;
   int changed = 0;
-  smartlist_t *descriptor_list = get_descriptor_list();
+  routerlist_t *rl = router_get_routerlist();
 
-  if (!descriptor_list)
-    return;
-
-  for (i = 0; i < smartlist_len(descriptor_list); ++i) {
+  for (i = 0; i < smartlist_len(rl->routers); ++i) {
     const char *msg;
-    routerinfo_t *ent = smartlist_get(descriptor_list, i);
+    routerinfo_t *ent = smartlist_get(rl->routers, i);
     router_status_t r = dirserv_router_get_status(ent, &msg);
     switch (r) {
       case FP_REJECT:
-        log(LOG_INFO, "Router '%s' is now rejected: %s",
+        info(LD_DIRSERV, "Router '%s' is now rejected: %s",
             ent->nickname, msg?msg:"");
-        routerinfo_free(ent);
-        smartlist_del(descriptor_list, i--);
+        routerlist_remove(rl, ent, i--, 0);
         changed = 1;
         break;
       case FP_NAMED:
         if (!ent->is_verified || !ent->is_named) {
-          log(LOG_INFO, "Router '%s' is now verified and named.", ent->nickname);
+          info(LD_DIRSERV,
+               "Router '%s' is now verified and named.", ent->nickname);
           ent->is_verified = ent->is_named = 1;
           changed = 1;
         }
         break;
       case FP_VALID:
         if (!ent->is_verified || ent->is_named) {
-          log(LOG_INFO, "Router '%s' is now verified.", ent->nickname);
+          info(LD_DIRSERV, "Router '%s' is now verified.", ent->nickname);
           ent->is_verified = 1;
           ent->is_named = 0;
           changed = 1;
@@ -508,7 +513,8 @@ directory_remove_invalid(void)
         break;
       case FP_INVALID:
         if (ent->is_verified || ent->is_named) {
-          log(LOG_INFO, "Router '%s' is no longer verified.", ent->nickname);
+          info(LD_DIRSERV,
+               "Router '%s' is no longer verified.", ent->nickname);
           ent->is_verified = ent->is_named = 0;
           changed = 1;
         }
@@ -526,21 +532,15 @@ directory_remove_invalid(void)
 char *
 dirserver_getinfo_unregistered(const char *question)
 {
-  int i;
   router_status_t r;
   smartlist_t *answerlist;
   char buf[1024];
   char *answer;
-  routerinfo_t *ent;
   int min_bw = atoi(question);
-  smartlist_t *descriptor_list = get_descriptor_list();
-
-  if (!descriptor_list)
-    return tor_strdup("");
+  routerlist_t *rl = router_get_routerlist();
 
   answerlist = smartlist_create();
-  for (i = 0; i < smartlist_len(descriptor_list); ++i) {
-    ent = smartlist_get(descriptor_list, i);
+  SMARTLIST_FOREACH(rl->routers, routerinfo_t *, ent, {
     r = dirserv_router_get_status(ent, NULL);
     if (ent->bandwidthcapacity >= (size_t)min_bw &&
         ent->bandwidthrate >= (size_t)min_bw &&
@@ -552,7 +552,7 @@ dirserver_getinfo_unregistered(const char *question)
                    ent->platform ? ent->platform : "");
       smartlist_add(answerlist, tor_strdup(buf));
     }
-  }
+  });
   answer = smartlist_join_strings(answerlist, "\r\n", 0, NULL);
   SMARTLIST_FOREACH(answerlist, char *, cp, tor_free(cp));
   smartlist_free(answerlist);
@@ -599,7 +599,7 @@ list_single_server_status(routerinfo_t *desc, int is_live)
     *cp++ = '=';
   }
   *cp++ = '$';
-  base16_encode(cp, HEX_DIGEST_LEN+1, desc->identity_digest,
+  base16_encode(cp, HEX_DIGEST_LEN+1, desc->cache_info.identity_digest,
                 DIGEST_LEN);
   return tor_strdup(buf);
 }
@@ -616,7 +616,7 @@ dirserv_thinks_router_is_reachable(routerinfo_t *router, time_t now)
   connection_t *conn;
   if (router_is_me(router) && !we_are_hibernating())
     return 1;
-  conn = connection_get_by_identity_digest(router->identity_digest,
+  conn = connection_get_by_identity_digest(router->cache_info.identity_digest,
                                            CONN_TYPE_OR);
   if (conn && conn->state == OR_CONN_STATE_OPEN)
     return get_options()->AssumeReachable ||
@@ -633,7 +633,7 @@ dirserv_thinks_router_is_blatantly_unreachable(routerinfo_t *router, time_t now)
   connection_t *conn;
   if (router->is_hibernating)
     return 0;
-  conn = connection_get_by_identity_digest(router->identity_digest,
+  conn = connection_get_by_identity_digest(router->cache_info.identity_digest,
                                            CONN_TYPE_OR);
   if (conn && conn->state == OR_CONN_STATE_OPEN &&
       now >= router->last_reachable + 2*REACHABLE_TIMEOUT &&
@@ -731,6 +731,7 @@ int
 dirserv_dump_directory_to_string(char **dir_out,
                                  crypto_pk_env_t *private_key)
 {
+  char *cp;
   char *router_status;
   char *identity_pkey; /* Identity key, DER64-encoded. */
   char *recommended_versions;
@@ -740,20 +741,17 @@ dirserv_dump_directory_to_string(char **dir_out,
   char *buf = NULL;
   size_t buf_len;
   size_t identity_pkey_len;
-  smartlist_t *descriptor_list = get_descriptor_list();
+  routerlist_t *rl = router_get_routerlist();
 
   tor_assert(dir_out);
   *dir_out = NULL;
 
-  if (!descriptor_list)
-    return -1;
-
-  if (list_server_status(descriptor_list, &router_status))
+  if (list_server_status(rl->routers, &router_status))
     return -1;
 
   if (crypto_pk_write_public_key_to_string(private_key,&identity_pkey,
                                            &identity_pkey_len)<0) {
-    log_fn(LOG_WARN,"write identity_pkey to string failed!");
+    warn(LD_BUG,"write identity_pkey to string failed!");
     return -1;
   }
 
@@ -764,8 +762,8 @@ dirserv_dump_directory_to_string(char **dir_out,
 
   buf_len = 2048+strlen(recommended_versions)+
     strlen(router_status);
-  SMARTLIST_FOREACH(descriptor_list, routerinfo_t *, ri,
-                    buf_len += ri->signed_descriptor_len);
+  SMARTLIST_FOREACH(rl->routers, routerinfo_t *, ri,
+                    buf_len += ri->cache_info.signed_descriptor_len+1);
   buf = tor_malloc(buf_len);
   /* We'll be comparing against buf_len throughout the rest of the
      function, though strictly speaking we shouldn't be able to exceed
@@ -785,9 +783,18 @@ dirserv_dump_directory_to_string(char **dir_out,
   tor_free(router_status);
   tor_free(identity_pkey);
 
-  SMARTLIST_FOREACH(descriptor_list, routerinfo_t *, ri,
-    if (strlcat(buf, ri->signed_descriptor, buf_len) >= buf_len)
-      goto truncated);
+  cp = buf + strlen(buf);
+  SMARTLIST_FOREACH(rl->routers, routerinfo_t *, ri,
+    {
+      size_t len = ri->cache_info.signed_descriptor_len;
+      if (cp+len+1 >= buf+buf_len)
+        goto truncated;
+      memcpy(cp, ri->cache_info.signed_descriptor, len);
+      cp += len;
+      *cp++ = '\n'; /* add an extra newline in case somebody was depending on
+                     * it. */
+    });
+  *cp = '\0';
 
   /* These multiple strlcat calls are inefficient, but dwarfed by the RSA
      signature.
@@ -800,7 +807,7 @@ dirserv_dump_directory_to_string(char **dir_out,
     goto truncated;
 
   if (router_get_dir_hash(buf,digest)) {
-    log_fn(LOG_WARN,"couldn't compute digest");
+    warn(LD_BUG,"couldn't compute digest");
     tor_free(buf);
     return -1;
   }
@@ -812,7 +819,7 @@ dirserv_dump_directory_to_string(char **dir_out,
   *dir_out = buf;
   return 0;
  truncated:
-  log_fn(LOG_WARN,"tried to exceed string length.");
+  warn(LD_BUG,"tried to exceed string length.");
   tor_free(buf);
   return -1;
 }
@@ -827,7 +834,7 @@ static cached_dir_t cached_runningrouters = { NULL, NULL, 0, 0, 0 };
 
 /* Used for other dirservers' v2 network statuses.  Map from hexdigest to
  * cached_dir_t. */
-static strmap_t *cached_v2_networkstatus = NULL;
+static digestmap_t *cached_v2_networkstatus = NULL;
 
 /** Possibly replace the contents of <b>d</b> with the value of
  * <b>directory</b> published on <b>when</b>, unless <b>when</b> is older than
@@ -840,21 +847,21 @@ set_cached_dir(cached_dir_t *d, char *directory, time_t when)
 {
   time_t now = time(NULL);
   if (when<=d->published) {
-    log_fn(LOG_INFO, "Ignoring old directory; not caching.");
+    info(LD_DIRSERV, "Ignoring old directory; not caching.");
     tor_free(directory);
   } else if (when>=now+ROUTER_MAX_AGE) {
-    log_fn(LOG_INFO, "Ignoring future directory; not caching.");
+    info(LD_DIRSERV, "Ignoring future directory; not caching.");
     tor_free(directory);
   } else {
     /* if (when>d->published && when<now+ROUTER_MAX_AGE) */
-    log_fn(LOG_DEBUG, "Caching directory.");
+    debug(LD_DIRSERV, "Caching directory.");
     tor_free(d->dir);
     d->dir = directory;
     d->dir_len = strlen(directory);
     tor_free(d->dir_z);
     if (tor_gzip_compress(&(d->dir_z), &(d->dir_z_len), d->dir, d->dir_len,
                           ZLIB_METHOD)) {
-      log_fn(LOG_WARN,"Error compressing cached directory");
+      warn(LD_BUG,"Error compressing cached directory");
     }
     d->published = when;
   }
@@ -891,26 +898,25 @@ dirserv_set_cached_directory(const char *directory, time_t published,
 }
 
 /** We've just received a v2 network-status for an authoritative directory
- * with fingerprint <b>fp</b> (hex digest, no spaces), published at
+ * with identity digest <b>identity</b> published at
  * <b>published</b>.  Store it so we can serve it to others.  If
  * <b>directory</b> is NULL, remove the entry with the given fingerprint from
  * the cache.
  */
 void
-dirserv_set_cached_networkstatus_v2(const char *directory, const char *fp,
+dirserv_set_cached_networkstatus_v2(const char *directory,
+                                    const char *identity,
                                     time_t published)
 {
   cached_dir_t *d;
   if (!cached_v2_networkstatus)
-    cached_v2_networkstatus = strmap_new();
+    cached_v2_networkstatus = digestmap_new();
 
-  tor_assert(strlen(fp) == HEX_DIGEST_LEN);
-
-  if (!(d = strmap_get(cached_v2_networkstatus, fp))) {
+  if (!(d = digestmap_get(cached_v2_networkstatus, identity))) {
     if (!directory)
       return;
     d = tor_malloc_zero(sizeof(cached_dir_t));
-    strmap_set(cached_v2_networkstatus, fp, d);
+    digestmap_set(cached_v2_networkstatus, identity, d);
   }
 
   tor_assert(d);
@@ -918,7 +924,7 @@ dirserv_set_cached_networkstatus_v2(const char *directory, const char *fp,
     set_cached_dir(d, tor_strdup(directory), published);
   } else {
     free_cached_dir(d);
-    strmap_remove(cached_v2_networkstatus, fp);
+    digestmap_remove(cached_v2_networkstatus, identity);
   }
 }
 
@@ -944,11 +950,11 @@ dirserv_pick_cached_dir_obj(cached_dir_t *cache_src,
     if (regenerate != NULL) {
       if (dirty && dirty + DIR_REGEN_SLACK_TIME < time(NULL)) {
         if (regenerate()) {
-          log_fn(LOG_ERR, "Couldn't generate %s?", name);
+          err(LD_BUG, "Couldn't generate %s?", name);
           exit(1);
         }
       } else {
-        log_fn(LOG_INFO, "The %s is still clean; reusing.", name);
+        info(LD_DIRSERV, "The %s is still clean; reusing.", name);
       }
     }
     return auth_src ? auth_src : cache_src;
@@ -1012,13 +1018,15 @@ dirserv_regenerate_directory(void)
 
   if (dirserv_dump_directory_to_string(&new_directory,
                                        get_identity_key())) {
-    log(LOG_WARN, "Error creating directory.");
+    warn(LD_BUG, "Error creating directory.");
     tor_free(new_directory);
     return -1;
   }
   set_cached_dir(&the_directory, new_directory, time(NULL));
-  log_fn(LOG_INFO,"New directory (size %d):\n%s",(int)the_directory.dir_len,
-         the_directory.dir);
+  info(LD_DIRSERV,"New directory (size %d) has been built.",
+       (int)the_directory.dir_len);
+  debug(LD_DIRSERV,"New directory (size %d):\n%s",(int)the_directory.dir_len,
+        the_directory.dir);
 
   the_directory_is_dirty = 0;
 
@@ -1044,14 +1052,14 @@ generate_runningrouters(void)
   crypto_pk_env_t *private_key = get_identity_key();
   char *identity_pkey; /* Identity key, DER64-encoded. */
   size_t identity_pkey_len;
-  smartlist_t *descriptor_list = get_descriptor_list();
+  routerlist_t *rl = router_get_routerlist();
 
-  if (list_server_status(descriptor_list, &router_status)) {
+  if (list_server_status(rl->routers, &router_status)) {
     goto err;
   }
   if (crypto_pk_write_public_key_to_string(private_key,&identity_pkey,
                                            &identity_pkey_len)<0) {
-    log_fn(LOG_WARN,"write identity_pkey to string failed!");
+    warn(LD_BUG,"write identity_pkey to string failed!");
     goto err;
   }
   format_iso_time(published, time(NULL));
@@ -1068,7 +1076,7 @@ generate_runningrouters(void)
   tor_free(router_status);
   tor_free(identity_pkey);
   if (router_get_runningrouters_hash(s,digest)) {
-    log_fn(LOG_WARN,"couldn't compute digest");
+    warn(LD_BUG,"couldn't compute digest");
     goto err;
   }
   if (router_append_dirobj_signature(s, len, digest, private_key)<0)
@@ -1155,19 +1163,14 @@ generate_v2_networkstatus(void)
   struct in_addr in;
   uint32_t addr;
   crypto_pk_env_t *private_key = get_identity_key();
-  smartlist_t *descriptor_list = get_descriptor_list();
+  routerlist_t *rl = router_get_routerlist();
   time_t now = time(NULL);
   int naming = options->NamingAuthoritativeDir;
   int versioning = options->VersioningAuthoritativeDir;
   const char *contact;
 
-  if (!descriptor_list) {
-    log_fn(LOG_WARN, "Couldn't get router list.");
-    goto done;
-  }
-
   if (resolve_my_address(options, &addr, &hostname)<0) {
-    log_fn(LOG_WARN, "Couldn't resolve my hostname");
+    warn(LD_NET, "Couldn't resolve my hostname");
     goto done;
   }
   in.s_addr = htonl(addr);
@@ -1180,12 +1183,12 @@ generate_v2_networkstatus(void)
 
   if (crypto_pk_write_public_key_to_string(private_key, &identity_pkey,
                                            &identity_pkey_len)<0) {
-    log_fn(LOG_WARN,"Writing public key to string failed.");
+    warn(LD_BUG,"Writing public key to string failed.");
     goto done;
   }
 
   if (crypto_pk_get_fingerprint(private_key, fingerprint, 0)<0) {
-    log_fn(LOG_ERR, "Error computing fingerprint");
+    err(LD_BUG, "Error computing fingerprint");
     goto done;
   }
 
@@ -1194,7 +1197,7 @@ generate_v2_networkstatus(void)
     contact = "(none)";
 
   len = 2048+strlen(client_versions)+strlen(server_versions)+identity_pkey_len*2;
-  len += (RS_ENTRY_LEN)*smartlist_len(descriptor_list) ;
+  len += (RS_ENTRY_LEN)*smartlist_len(rl->routers);
 
   status = tor_malloc(len);
   tor_snprintf(status, len,
@@ -1219,12 +1222,13 @@ generate_v2_networkstatus(void)
   outp = status + strlen(status);
   endp = status + len;
 
-  SMARTLIST_FOREACH(descriptor_list, routerinfo_t *, ri, {
+  SMARTLIST_FOREACH(rl->routers, routerinfo_t *, ri, {
       int f_exit = router_is_general_exit(ri);
       int f_stable = !router_is_unreliable(ri, 1, 0);
       int f_fast = !router_is_unreliable(ri, 0, 1);
       int f_running;
-      int f_authority = router_digest_is_trusted_dir(ri->identity_digest);
+      int f_authority = router_digest_is_trusted_dir(
+                                      ri->cache_info.identity_digest);
       int f_named = naming && ri->is_named;
       int f_valid = ri->is_verified;
       char identity64[BASE64_DIGEST_LEN+1];
@@ -1234,10 +1238,10 @@ generate_v2_networkstatus(void)
       }
       f_running = ri->is_running;
 
-      format_iso_time(published, ri->published_on);
+      format_iso_time(published, ri->cache_info.published_on);
 
-      digest_to_base64(identity64, ri->identity_digest);
-      digest_to_base64(digest64, ri->signed_descriptor_digest);
+      digest_to_base64(identity64, ri->cache_info.identity_digest);
+      digest_to_base64(digest64, ri->cache_info.signed_descriptor_digest);
 
       in.s_addr = htonl(ri->addr);
       tor_inet_ntoa(&in, ipaddr, sizeof(ipaddr));
@@ -1259,25 +1263,25 @@ generate_v2_networkstatus(void)
                        f_stable?" Stable":"",
                        f_running?" Running":"",
                        f_valid?" Valid":"")<0) {
-        log_fn(LOG_WARN, "Unable to print router status.");
+        warn(LD_BUG, "Unable to print router status.");
         goto done;
-        }
+      }
       outp += strlen(outp);
     });
 
   if (tor_snprintf(outp, endp-outp, "directory-signature %s\n",
                    get_options()->Nickname)<0) {
-    log_fn(LOG_WARN, "Unable to write signature line.");
+    warn(LD_BUG, "Unable to write signature line.");
     goto done;
   }
 
   if (router_get_networkstatus_v2_hash(status, digest)<0) {
-    log_fn(LOG_WARN, "Unable to hash network status");
+    warn(LD_BUG, "Unable to hash network status");
     goto done;
   }
 
   if (router_append_dirobj_signature(outp,endp-outp,digest,private_key)<0) {
-    log_fn(LOG_WARN, "Unable to sign router status.");
+    warn(LD_BUG, "Unable to sign router status.");
     goto done;
   }
 
@@ -1312,7 +1316,7 @@ dirserv_get_networkstatus_v2(smartlist_t *result,
   tor_assert(result);
 
   if (!cached_v2_networkstatus)
-    cached_v2_networkstatus = strmap_new();
+    cached_v2_networkstatus = digestmap_new();
 
   if (!(strcmp(key,"authority"))) {
     if (get_options()->AuthoritativeDir) {
@@ -1322,94 +1326,116 @@ dirserv_get_networkstatus_v2(smartlist_t *result,
                                     the_v2_networkstatus_is_dirty,
                                     generate_v2_networkstatus,
                                     "network status list", 0);
-      log_fn(LOG_WARN, "Unable to generate an authoritative network stautus.");
       if (d)
         smartlist_add(result, d);
+      else
+        warn(LD_BUG,"Unable to generate an authoritative network status.");
     }
   } else if (!strcmp(key, "all")) {
-    strmap_iter_t *iter = strmap_iter_init(cached_v2_networkstatus);
-    while (!strmap_iter_done(iter)) {
-      const char *fp;
+    digestmap_iter_t *iter = digestmap_iter_init(cached_v2_networkstatus);
+    while (!digestmap_iter_done(iter)) {
+      const char *ident;
       void *val;
-      strmap_iter_get(iter, &fp, &val);
+      digestmap_iter_get(iter, &ident, &val);
       smartlist_add(result, val);
-      iter = strmap_iter_next(cached_v2_networkstatus, iter);
+      iter = digestmap_iter_next(cached_v2_networkstatus, iter);
     }
     if (smartlist_len(result) == 0)
-      log_fn(LOG_WARN, "Client requested 'all' network status objects; we have none.");
+      warn(LD_DIRSERV,
+           "Client requested 'all' network status objects; we have none.");
   } else if (!strcmpstart(key, "fp/")) {
-    smartlist_t *hexdigests = smartlist_create();
-    dir_split_resource_into_fingerprints(key+3, hexdigests, NULL);
-    SMARTLIST_FOREACH(hexdigests, char *, cp,
+    smartlist_t *digests = smartlist_create();
+    dir_split_resource_into_fingerprints(key+3, digests, NULL, 1);
+    SMARTLIST_FOREACH(digests, char *, cp,
         {
           cached_dir_t *cached;
-          tor_strupper(cp);
-          if (router_fingerprint_is_me(cp) &&
+          if (router_digest_is_me(cp) &&
               get_options()->AuthoritativeDir &&
               the_v2_networkstatus_is_dirty &&
               the_v2_networkstatus_is_dirty + DIR_REGEN_SLACK_TIME < time(NULL))
             generate_v2_networkstatus();
-          cached = strmap_get(cached_v2_networkstatus, cp);
+          cached = digestmap_get(cached_v2_networkstatus, cp);
           if (cached) {
             smartlist_add(result, cached);
           } else {
-            log_fn(LOG_INFO, "Don't know about any network status with fingerprint '%s'", cp);
+            char hexbuf[HEX_DIGEST_LEN+1];
+            base16_encode(hexbuf, sizeof(hexbuf), cp, DIGEST_LEN);
+            info(LD_DIRSERV, "Don't know about any network status with fingerprint '%s'", hexbuf);
           }
           tor_free(cp);
         });
-    smartlist_free(hexdigests);
+    smartlist_free(digests);
   }
   return 0;
 }
 
-/** Add a routerinfo_t to <b>descs_out</b> for each router matching
- * <b>key</b>.  The key should be either "/tor/server/authority" for our own
- * routerinfo; "/tor/server/all" for all the routerinfos we have,
- * concatenated; or "/tor/server/FP" where FP is a plus-separated sequence of
- * hex identity digests.
+/** Add a signed_descriptor_t to <b>descs_out</b> for each router matching
+ * <b>key</b>.  The key should be either
+ *   - "/tor/server/authority" for our own routerinfo;
+ *   - "/tor/server/all" for all the routerinfos we have, concatenated;
+ *   - "/tor/server/fp/FP" where FP is a plus-separated sequence of
+ *     hex identity digests; or
+ *   - "/tor/server/d/D" where D is a plus-separated sequence
+ *     of server descriptor digests, in hex.
+ *
+ * Return 0 if we found some matching descriptors, or -1 if we do not
+ * have any descriptors, no matching descriptors, or if we did not
+ * recognize the key (URL).
+ * If -1 is returned *<b>msg</b> will be set to an appropriate error
+ * message.
  */
-void
-dirserv_get_routerdescs(smartlist_t *descs_out, const char *key)
+int
+dirserv_get_routerdescs(smartlist_t *descs_out, const char *key,
+  const char **msg)
 {
-  smartlist_t *complete_list = get_descriptor_list();
-  if (!complete_list)
-    return;
+  *msg = NULL;
 
   if (!strcmp(key, "/tor/server/all")) {
-    smartlist_add_all(descs_out, complete_list);
+    routerlist_t *rl = router_get_routerlist();
+    SMARTLIST_FOREACH(rl->routers, routerinfo_t *, r,
+                      smartlist_add(descs_out, &(r->cache_info)));
   } else if (!strcmp(key, "/tor/server/authority")) {
     routerinfo_t *ri = router_get_my_routerinfo();
     if (ri)
-      smartlist_add(descs_out, ri);
+      smartlist_add(descs_out, &(ri->cache_info));
+  } else if (!strcmpstart(key, "/tor/server/d/")) {
+    smartlist_t *digests = smartlist_create();
+    key += strlen("/tor/server/d/");
+    dir_split_resource_into_fingerprints(key, digests, NULL, 1);
+    SMARTLIST_FOREACH(digests, const char *, d,
+       {
+         signed_descriptor_t *sd = router_get_by_descriptor_digest(d);
+         if (sd)
+           smartlist_add(descs_out,sd);
+       });
+    SMARTLIST_FOREACH(digests, char *, d, tor_free(d));
+    smartlist_free(digests);
   } else if (!strcmpstart(key, "/tor/server/fp/")) {
-    smartlist_t *hexdigests = smartlist_create();
     smartlist_t *digests = smartlist_create();
     key += strlen("/tor/server/fp/");
-    dir_split_resource_into_fingerprints(key, hexdigests, NULL);
-    SMARTLIST_FOREACH(hexdigests, char *, cp,
-                      {
-                        char *d;
-                        if (strlen(cp) != HEX_DIGEST_LEN)
-                          continue;
-                        d = tor_malloc_zero(DIGEST_LEN);
-                        base16_decode(d, DIGEST_LEN, cp, HEX_DIGEST_LEN);
-                        tor_free(cp);
-                        smartlist_add(digests, d);
-                      });
-    smartlist_free(hexdigests);
+    dir_split_resource_into_fingerprints(key, digests, NULL, 1);
     SMARTLIST_FOREACH(digests, const char *, d,
        {
          if (router_digest_is_me(d)) {
-           smartlist_add(descs_out, router_get_my_routerinfo());
+           smartlist_add(descs_out, &(router_get_my_routerinfo()->cache_info));
          } else {
            routerinfo_t *ri = router_get_by_digest(d);
            if (ri)
-             smartlist_add(descs_out,ri);
+             smartlist_add(descs_out, &(ri->cache_info));
          }
        });
     SMARTLIST_FOREACH(digests, char *, d, tor_free(d));
     smartlist_free(digests);
+  } else {
+    *msg = "Key not recognized";
+    return -1;
   }
+
+  if (!smartlist_len(descs_out)) {
+    *msg = "Servers unavailable";
+    return -1;
+  }
+  return 0;
 }
 
 /** Called when a TLS handshake has completed successfully with a
@@ -1430,39 +1456,36 @@ dirserv_orconn_tls_done(const char *address,
                         int as_advertised)
 {
   int i;
-  smartlist_t *descriptor_list = get_descriptor_list();
+  routerlist_t *rl = router_get_routerlist();
   tor_assert(address);
   tor_assert(digest_rcvd);
   tor_assert(nickname_rcvd);
 
-  if (!descriptor_list)
-    return;
-
   // XXXXNM We should really have a better solution here than dropping
   // XXXXNM whole routers; otherwise, they come back way too easily.
-  for (i = 0; i < smartlist_len(descriptor_list); ++i) {
-    routerinfo_t *ri = smartlist_get(descriptor_list, i);
+  for (i = 0; i < smartlist_len(rl->routers); ++i) {
+    routerinfo_t *ri = smartlist_get(rl->routers, i);
     int drop = 0;
     if (strcasecmp(address, ri->address) || or_port != ri->or_port)
       continue;
     if (!ri->is_verified) {
       /* We have a router at the same address! */
       if (strcasecmp(ri->nickname, nickname_rcvd)) {
-        log_fn(LOG_NOTICE, "Dropping descriptor: nickname '%s' does not match nickname '%s' in cert from %s:%d",
+        notice(LD_DIRSERV, "Dropping descriptor: nickname '%s' does not match nickname '%s' in cert from %s:%d",
                ri->nickname, nickname_rcvd, address, or_port);
         drop = 1;
-      } else if (memcmp(ri->identity_digest, digest_rcvd, DIGEST_LEN)) {
-        log_fn(LOG_NOTICE, "Dropping descriptor: identity key does not match key in cert from %s:%d",
+      } else if (memcmp(ri->cache_info.identity_digest, digest_rcvd,
+                        DIGEST_LEN)) {
+        notice(LD_DIRSERV, "Dropping descriptor: identity key does not match key in cert from %s:%d",
                address, or_port);
         drop = 1;
       }
     }
     if (drop) {
-      routerinfo_free(ri);
-      smartlist_del(descriptor_list, i--);
+      routerlist_remove(rl, ri, i--, 0);
       directory_set_dirty();
     } else { /* correct nickname and digest. mark this router reachable! */
-      log_fn(LOG_INFO,"Found router %s to be reachable. Yay.", ri->nickname);
+      info(LD_DIRSERV,"Found router %s to be reachable. Yay.", ri->nickname);
       ri->last_reachable = time(NULL);
       ri->num_unreachable_notifications = 0;
     }
@@ -1487,10 +1510,11 @@ dirserv_free_all(void)
     addr_policy_free(authdir_invalid_policy);
   clear_cached_dir(&the_directory);
   clear_cached_dir(&the_runningrouters);
+  clear_cached_dir(&the_v2_networkstatus);
   clear_cached_dir(&cached_directory);
   clear_cached_dir(&cached_runningrouters);
   if (cached_v2_networkstatus) {
-    strmap_free(cached_v2_networkstatus, free_cached_dir);
+    digestmap_free(cached_v2_networkstatus, free_cached_dir);
     cached_v2_networkstatus = NULL;
   }
 }
