@@ -2,7 +2,7 @@
  * Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: or.h,v 1.710 2005/10/07 19:25:01 arma Exp $ */
+/* $Id: or.h,v 1.732 2005/11/11 19:25:30 arma Exp $ */
 
 /**
  * \file or.h
@@ -11,7 +11,7 @@
 
 #ifndef __OR_H
 #define __OR_H
-#define OR_H_ID "$Id: or.h,v 1.710 2005/10/07 19:25:01 arma Exp $"
+#define OR_H_ID "$Id: or.h,v 1.732 2005/11/11 19:25:30 arma Exp $"
 
 #include "orconfig.h"
 #ifdef MS_WINDOWS
@@ -604,6 +604,9 @@ struct connection_t {
   unsigned has_sent_end:1; /**< For debugging; only used on edge connections.
                          * Set once we've set the stream end,
                          * and check in circuit_about_to_close_connection(). */
+  /** For control connections only. If set, we send extended info with control
+   * events as appropriate. */
+  unsigned int control_events_are_extended:1;
 
   int s; /**< Our socket; -1 if this connection is closed. */
   int poll_index; /* XXXX rename. */
@@ -732,11 +735,18 @@ typedef struct cached_dir_t {
   time_t published; /**< When was this object published */
 } cached_dir_t;
 
+/** Information need to cache an onion router's descriptor. */
+typedef struct signed_descriptor_t {
+  char *signed_descriptor;
+  size_t signed_descriptor_len;
+  char signed_descriptor_digest[DIGEST_LEN];
+  char identity_digest[DIGEST_LEN];
+  time_t published_on;
+} signed_descriptor_t;
+
 /** Information about another onion router in the network. */
 typedef struct {
-  char *signed_descriptor; /**< The original signed descriptor for this router*/
-  size_t signed_descriptor_len; /**< The length of signed_descriptor */
-  char signed_descriptor_digest[DIGEST_LEN]; /**< The digest of the signed descriptor. */
+  signed_descriptor_t cache_info;
 
   char *address; /**< Location of OR: either a hostname or an IP address. */
   char *nickname; /**< Human-readable OR name. */
@@ -745,12 +755,8 @@ typedef struct {
   uint16_t or_port; /**< Port for OR-to-OR and OP-to-OR connections. */
   uint16_t dir_port; /**< Port for HTTP directory connections. */
 
-  time_t published_on; /**< When was the information in this routerinfo_t
-                        * published? */
-
   crypto_pk_env_t *onion_pkey; /**< Public RSA key for onions. */
   crypto_pk_env_t *identity_pkey;  /**< Public RSA key for signing. */
-  char identity_digest[DIGEST_LEN]; /**< Digest of identity key */
 
   char *platform; /**< What software/operating system is this OR using? */
 
@@ -777,6 +783,12 @@ typedef struct {
                                */
   unsigned int is_named:1; /**< Do we believe the nickname that this OR gives
                             * us? */
+  unsigned int xx_is_recognized:1; /**< Temporary: do we think that this
+                                    * descriptor's digest is recognized?
+                                    */
+  unsigned int xx_is_extra_new:1; /**< Temporary: do we think that this
+                                   * descriptor's digest is recognized?
+                                   */
 
   /* The below items are used only by authdirservers for
    * reachability testing. */
@@ -860,8 +872,16 @@ typedef struct networkstatus_t {
 
 /** Contents of a directory of onion routers. */
 typedef struct {
-  /** List of routerinfo_t. */
+  /** Map from server identity digest to a member of routers. */
+  digestmap_t *identity_map;
+  /** Map from server descriptor digest to a signed_descritptor_t from
+   * routers or old_routers. */
+  digestmap_t *desc_digest_map;
+  /** List of routerinfo_t for all currently live routers we know. */
   smartlist_t *routers;
+  /** List of signed_descriptor_t for older router descriptors we're
+   * caching. */
+  smartlist_t *old_routers;
 } routerlist_t;
 
 /** Information on router used when extending a circuit.  (We don't need a
@@ -1052,8 +1072,11 @@ struct circuit_t {
    */
   crypt_path_t *cpath;
 
-  /** For storage while passing to cpuworker, or while n_conn is pending. */
-  char onionskin[ONIONSKIN_CHALLENGE_LEN];
+  /** For storage while passing to cpuworker (state
+    * CIRCUIT_STATE_ONIONSKIN_PENDING), or while n_conn is pending
+    * (state CIRCUIT_STATE_OR_WAIT). When defined, it is always
+    * length ONIONSKIN_CHALLENGE_LEN. */
+  char *onionskin;
 
   char handshake_digest[DIGEST_LEN]; /**< Stores KH for intermediate hops. */
 
@@ -1065,7 +1088,7 @@ struct circuit_t {
   uint8_t purpose; /**< Why are we creating this circuit? */
 
   /**
-   * The rend_query field holds y portion of y.onion (nul-terminated)
+   * The rend_query field holds the y portion of y.onion (nul-terminated)
    * if purpose is C_INTRODUCING or C_ESTABLISH_REND, or is a C_GENERAL
    * for a hidden service, or is S_*.
    */
@@ -1165,11 +1188,11 @@ typedef struct {
   config_line_t *SocksPolicy; /**< Lists of socks policy components */
   config_line_t *DirPolicy; /**< Lists of dir policy components */
   /** Addresses to bind for listening for SOCKS connections. */
-  config_line_t *SocksBindAddress;
+  config_line_t *SocksListenAddress;
   /** Addresses to bind for listening for OR connections. */
-  config_line_t *ORBindAddress;
+  config_line_t *ORListenAddress;
   /** Addresses to bind for listening for directory connections. */
-  config_line_t *DirBindAddress;
+  config_line_t *DirListenAddress;
   /** Local address to bind outbound sockets */
   char *OutboundBindAddress;
   /** Directory server only: which versions of
@@ -1199,8 +1222,6 @@ typedef struct {
   int NoPublish; /**< Boolean: should we never publish a descriptor? */
   int ConnLimit; /**< Requested maximum number of simultaneous connections. */
   int _ConnLimit; /**< Actual maximum number of simultaneous connections. */
-  int IgnoreVersion; /**< If true, run no matter what versions of Tor the
-                      * directory recommends. */
   int RunAsDaemon; /**< If true, run in the background. (Unix only) */
   int FascistFirewall; /**< Whether to prefer ORs reachable on open ports. */
   smartlist_t *FirewallPorts; /**< Which ports our firewall allows (strings). */
@@ -1283,6 +1304,9 @@ typedef struct {
                            * long do we wait before exiting? */
   int SafeLogging; /**< Boolean: are we allowed to log sensitive strings
                     * such as addresses (0), or do we scrub them first (1)? */
+#define LOG_PROTOCOL_WARN (get_options()->ProtocolWarnings ? LOG_WARN : LOG_INFO)
+  int ProtocolWarnings; /**< Boolean: when other parties screw up the Tor
+                         * protocol, is it a warn or an info in our logs? */
   int HardwareAccel; /**< Boolean: Should we enable OpenSSL hardware
                       * acceleration where available? */
   int UseHelperNodes; /**< Boolean: Do we try to enter from a smallish number
@@ -1346,14 +1370,15 @@ const char *_buf_peek_raw_buffer(const buf_t *buf);
 int read_to_buf(int s, size_t at_most, buf_t *buf, int *reached_eof);
 int read_to_buf_tls(tor_tls_t *tls, size_t at_most, buf_t *buf);
 
-int flush_buf(int s, buf_t *buf, size_t *buf_flushlen);
-int flush_buf_tls(tor_tls_t *tls, buf_t *buf, size_t *buf_flushlen);
+int flush_buf(int s, buf_t *buf, size_t sz, size_t *buf_flushlen);
+int flush_buf_tls(tor_tls_t *tls, buf_t *buf, size_t sz, size_t *buf_flushlen);
 
 int write_to_buf(const char *string, size_t string_len, buf_t *buf);
 int fetch_from_buf(char *string, size_t string_len, buf_t *buf);
 int fetch_from_buf_http(buf_t *buf,
                         char **headers_out, size_t max_headerlen,
-                        char **body_out, size_t *body_used, size_t max_bodylen);
+                        char **body_out, size_t *body_used, size_t max_bodylen,
+                        int force_complete);
 int fetch_from_buf_socks(buf_t *buf, socks_request_t *req);
 int fetch_from_buf_control0(buf_t *buf, uint32_t *len_out, uint16_t *type_out,
                             char **body_out, int check_for_v1);
@@ -1364,7 +1389,7 @@ void assert_buf_ok(buf_t *buf);
 /********************************* circuitbuild.c **********************/
 
 char *circuit_list_path(circuit_t *circ, int verbose);
-void circuit_log_path(int severity, circuit_t *circ);
+void circuit_log_path(int severity, unsigned int domain, circuit_t *circ);
 void circuit_rep_hist_note_result(circuit_t *circ);
 void circuit_dump_by_conn(connection_t *conn, int severity);
 circuit_t *circuit_init(uint8_t purpose, int need_uptime,
@@ -1410,6 +1435,7 @@ void circuit_set_circid_orconn(circuit_t *circ, uint16_t id,
 void circuit_close_all_marked(void);
 circuit_t *circuit_new(uint16_t p_circ_id, connection_t *p_conn);
 circuit_t *circuit_get_by_circid_orconn(uint16_t circ_id, connection_t *conn);
+int circuit_id_used_on_conn(uint16_t circ_id, connection_t *conn);
 circuit_t *circuit_get_by_edge_conn(connection_t *conn);
 circuit_t *circuit_get_by_conn(connection_t *conn);
 circuit_t *circuit_get_by_global_id(uint32_t id);
@@ -1417,8 +1443,9 @@ circuit_t *circuit_get_by_rend_query_and_purpose(const char *rend_query, uint8_t
 circuit_t *circuit_get_next_by_pk_and_purpose(circuit_t *start,
                                          const char *digest, uint8_t purpose);
 circuit_t *circuit_get_rendezvous(const char *cookie);
-circuit_t *circuit_get_clean_open(uint8_t purpose, int need_uptime,
-                                  int need_capacity, int internal);
+circuit_t *circuit_find_to_cannibalize(uint8_t purpose, extend_info_t *info,
+                                       int need_uptime,
+                                       int need_capacity, int internal);
 void circuit_mark_all_unused_circs(void);
 void circuit_expire_all_dirty_circs(void);
 void _circuit_mark_for_close(circuit_t *circ, int line, const char *file);
@@ -1480,6 +1507,7 @@ int options_init_logs(or_options_t *options, int validate_only);
 int config_parse_addr_policy(config_line_t *cfg,
                              addr_policy_t **dest,
                              int assume_action);
+int config_cmp_addr_policies(addr_policy_t *a, addr_policy_t *b);
 void options_append_default_exit_policy(addr_policy_t **policy);
 void addr_policy_free(addr_policy_t *p);
 int option_is_recognized(const char *key);
@@ -1521,6 +1549,7 @@ int connection_connect(connection_t *conn, char *address, uint32_t addr, uint16_
 int retry_all_listeners(int force, smartlist_t *replaced_conns,
                         smartlist_t *new_conns);
 
+int connection_bucket_write_limit(connection_t *conn);
 void connection_bucket_init(void);
 void connection_bucket_refill(struct timeval *now);
 
@@ -1704,7 +1733,7 @@ int control_event_circuit_status(circuit_t *circ, circuit_status_event_t e);
 int control_event_stream_status(connection_t *conn, stream_status_event_t e);
 int control_event_or_conn_status(connection_t *conn, or_conn_status_event_t e);
 int control_event_bandwidth_used(uint32_t n_read, uint32_t n_written);
-void control_event_logmsg(int severity, const char *msg);
+void control_event_logmsg(int severity, unsigned int domain, const char *msg);
 int control_event_descriptors_changed(smartlist_t *routers);
 int control_event_address_mapped(const char *from, const char *to,time_t expires);
 
@@ -1743,7 +1772,8 @@ void connection_dir_request_failed(connection_t *conn);
 void parse_dir_policy(void);
 void free_dir_policy(void);
 int dir_split_resource_into_fingerprints(const char *resource,
-                               smartlist_t *fp_out, int *compresseed_out);
+                                    smartlist_t *fp_out, int *compresseed_out,
+                                    int decode_hex);
 
 /********************************* dirserv.c ***************************/
 
@@ -1765,10 +1795,12 @@ size_t dirserv_get_directory(const char **cp, int compress);
 size_t dirserv_get_runningrouters(const char **rr, int compress);
 void dirserv_set_cached_directory(const char *directory, time_t when,
                                   int is_running_routers);
-void dirserv_set_cached_networkstatus_v2(const char *directory, const char *fp,
+void dirserv_set_cached_networkstatus_v2(const char *directory,
+                                         const char *identity,
                                          time_t published);
 int dirserv_get_networkstatus_v2(smartlist_t *result, const char *key);
-void dirserv_get_routerdescs(smartlist_t *descs_out, const char *key);
+int dirserv_get_routerdescs(smartlist_t *descs_out, const char *key,
+                            const char **msg);
 void dirserv_orconn_tls_done(const char *address,
                              uint16_t or_port,
                              const char *digest_rcvd,
@@ -2048,6 +2080,7 @@ void router_upload_dir_desc_to_dirservers(int force);
 void mark_my_descriptor_dirty_if_older_than(time_t when);
 void mark_my_descriptor_dirty(void);
 void check_descriptor_bandwidth_changed(time_t now);
+void check_descriptor_ipaddress_changed(time_t now);
 int router_compare_to_my_exit_policy(connection_t *conn);
 routerinfo_t *router_get_my_routerinfo(void);
 const char *router_get_my_descriptor(void);
@@ -2118,17 +2151,20 @@ routerinfo_t *router_get_by_nickname(const char *nickname,
                                      int warn_if_unnamed);
 routerinfo_t *router_get_by_hexdigest(const char *hexdigest);
 routerinfo_t *router_get_by_digest(const char *digest);
+signed_descriptor_t *router_get_by_descriptor_digest(const char *digest);
 int router_digest_is_trusted_dir(const char *digest);
-void router_get_routerlist(routerlist_t **prouterlist);
+routerlist_t *router_get_routerlist(void);
 void routerlist_reset_warnings(void);
 void routerlist_free(routerlist_t *routerlist);
+void routerlist_remove(routerlist_t *rl, routerinfo_t *ri, int idx,
+                       int make_old);
 void routerinfo_free(routerinfo_t *router);
 void routerstatus_free(routerstatus_t *routerstatus);
 void networkstatus_free(networkstatus_t *networkstatus);
 void routerlist_free_all(void);
 routerinfo_t *routerinfo_copy(const routerinfo_t *router);
 void router_mark_as_down(const char *digest);
-void routerlist_remove_old_routers(int age);
+void routerlist_remove_old_routers(void);
 void networkstatus_list_clean(time_t now);
 int router_add_to_routerlist(routerinfo_t *router, const char **msg,
                              int from_cache);
@@ -2156,12 +2192,14 @@ void update_networkstatus_downloads(time_t now);
 void update_router_descriptor_downloads(time_t now);
 void routers_update_all_from_networkstatus(void);
 void routers_update_status_from_networkstatus(smartlist_t *routers,
-                                              int reset_failures);
+                                              int reset_failures,
+                                              int assume_recognized);
 smartlist_t *router_list_superseded(void);
 int router_have_minimum_dir_info(void);
 void networkstatus_list_update_recent(time_t now);
 void router_reset_descriptor_download_failures(void);
 void router_reset_status_download_failures(void);
+int router_differences_are_cosmetic(routerinfo_t *r1, routerinfo_t *r2);
 
 /********************************* routerparse.c ************************/
 
