@@ -1,8 +1,8 @@
 /* Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: test.c,v 1.207 2005/09/30 20:47:58 nickm Exp $ */
-const char test_c_id[] = "$Id: test.c,v 1.207 2005/09/30 20:47:58 nickm Exp $";
+/* $Id: test.c,v 1.211 2005/11/05 20:15:27 nickm Exp $ */
+const char test_c_id[] = "$Id: test.c,v 1.211 2005/11/05 20:15:27 nickm Exp $";
 
 /**
  * \file test.c
@@ -919,7 +919,7 @@ test_gzip(void)
     test_assert(!memcmp(buf2, "\037\213", 2)); /* Gzip magic. */
     test_eq(detect_compression_method(buf2, len1), GZIP_METHOD);
 
-    test_assert(!tor_gzip_uncompress(&buf3, &len2, buf2, len1, GZIP_METHOD));
+    test_assert(!tor_gzip_uncompress(&buf3, &len2, buf2, len1, GZIP_METHOD, 1));
     test_assert(buf3);
     test_streq(buf1,buf3);
 
@@ -933,20 +933,40 @@ test_gzip(void)
   test_assert(!memcmp(buf2, "\x78\xDA", 2)); /* deflate magic. */
   test_eq(detect_compression_method(buf2, len1), ZLIB_METHOD);
 
-  test_assert(!tor_gzip_uncompress(&buf3, &len2, buf2, len1, ZLIB_METHOD));
+  test_assert(!tor_gzip_uncompress(&buf3, &len2, buf2, len1, ZLIB_METHOD, 1));
   test_assert(buf3);
   test_streq(buf1,buf3);
 
+  /* Check whether we can uncompress concatenated, compresed strings. */
   tor_free(buf3);
   buf2 = tor_realloc(buf2, len1*2);
   memcpy(buf2+len1, buf2, len1);
-  test_assert(!tor_gzip_uncompress(&buf3, &len2, buf2, len1*2, ZLIB_METHOD));
+  test_assert(!tor_gzip_uncompress(&buf3, &len2, buf2, len1*2, ZLIB_METHOD, 1));
   test_eq(len2, (strlen(buf1)+1)*2);
   test_memeq(buf3,
              "AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAZAAAAAAAAAAAAAAAAAAAZ\0"
              "AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAZAAAAAAAAAAAAAAAAAAAZ\0",
              (strlen(buf1)+1)*2);
 
+  tor_free(buf1);
+  tor_free(buf2);
+  tor_free(buf3);
+
+  /* Check whether we can uncompress partial strings. */
+  buf1 = tor_strdup("String with low redundancy that won't be compressed much.");
+  test_assert(!tor_gzip_compress(&buf2, &len1, buf1, strlen(buf1)+1,ZLIB_METHOD));
+  tor_assert(len1>16);
+  /* when we allow an uncomplete string, we should succeed.*/
+  tor_assert(!tor_gzip_uncompress(&buf3, &len2, buf2, len1-16, ZLIB_METHOD, 0));
+  buf3[len2]='\0';
+  tor_assert(len2 > 5);
+  tor_assert(!strcmpstart(buf1, buf3));
+
+  /* when we demand a complete string, this must fail. */
+  tor_free(buf3);
+  tor_assert(tor_gzip_uncompress(&buf3, &len2, buf2, len1-16, ZLIB_METHOD, 1));
+  tor_assert(!buf3);
+
   tor_free(buf2);
   tor_free(buf3);
   tor_free(buf1);
@@ -1167,7 +1187,7 @@ test_dir_format(void)
   memset(&r2,0,sizeof(r2));
   r1.address = tor_strdup("18.244.0.1");
   r1.addr = 0xc0a80001u; /* 192.168.0.1 */
-  r1.published_on = 0;
+  r1.cache_info.published_on = 0;
   r1.or_port = 9000;
   r1.dir_port = 9003;
   r1.onion_pkey = pk1;
@@ -1193,7 +1213,7 @@ test_dir_format(void)
   r2.address = tor_strdup("1.1.1.1");
   r2.addr = 0x0a030201u; /* 10.3.2.1 */
   r2.platform = tor_strdup(platform);
-  r2.published_on = 5;
+  r2.cache_info.published_on = 5;
   r2.or_port = 9005;
   r2.dir_port = 0;
   r2.onion_pkey = pk2;
@@ -1294,8 +1314,8 @@ test_dir_format(void)
     add_fingerprint_to_dir("Fred", buf, fingerprint_list);
   }
   /* Make sure routers aren't too far in the past any more. */
-  r1.published_on = time(NULL);
-  r2.published_on = time(NULL)-3*60*60;
+  r1.cache_info.published_on = time(NULL);
+  r2.cache_info.published_on = time(NULL)-3*60*60;
   test_assert(router_dump_router_to_string(buf, 2048, &r1, pk2)>0);
   test_eq(dirserv_add_descriptor(buf,&m), 2);
   test_assert(router_dump_router_to_string(buf, 2048, &r2, pk1)>0);
